from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class RfOffCls:
	"""RfOff commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("rfOff", core, parent)

	# noinspection PyTypeChecker
	def get_mode(self) -> enums.PowRfOffMode:
		"""SCPI: [SOURce<HW>]:POWer:ATTenuation:RFOFf:MODE \n
		Snippet: value: enums.PowRfOffMode = driver.source.power.attenuation.rfOff.get_mode() \n
		Determines the attenuator's state after the instrument is switched on. \n
			:return: mode: MAX| FATTenuated| FIXed| UNCHanged MAX = FATTenuated Sets attenuation to maximum when the RF signal is switched off. This setting is recommended for applications that require a high level of noise suppression. FIXed = UNCHanged Retains the current setting and keeps the output impedance constant during RF off.
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:POWer:ATTenuation:RFOFf:MODE?')
		return Conversions.str_to_scalar_enum(response, enums.PowRfOffMode)

	def set_mode(self, mode: enums.PowRfOffMode) -> None:
		"""SCPI: [SOURce<HW>]:POWer:ATTenuation:RFOFf:MODE \n
		Snippet: driver.source.power.attenuation.rfOff.set_mode(mode = enums.PowRfOffMode.FIXed) \n
		Determines the attenuator's state after the instrument is switched on. \n
			:param mode: MAX| FATTenuated| FIXed| UNCHanged MAX = FATTenuated Sets attenuation to maximum when the RF signal is switched off. This setting is recommended for applications that require a high level of noise suppression. FIXed = UNCHanged Retains the current setting and keeps the output impedance constant during RF off.
		"""
		param = Conversions.enum_scalar_to_str(mode, enums.PowRfOffMode)
		self._core.io.write(f'SOURce<HwInstance>:POWer:ATTenuation:RFOFf:MODE {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class SumCls:
	"""Sum commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("sum", core, parent)

	def get_pep(self) -> float:
		"""SCPI: [SOURce<HW>]:AWGN:POWer:SUM:PEP \n
		Snippet: value: float = driver.source.awgn.power.sum.get_pep() \n
		In Additive Noise and CW Interferer (SOUR:AWGN:MODE ADD|CW) modes, queries the peak envelope power of the overall signal
		comprised of noise signal plus useful signal. \n
			:return: pep: float Range: -145 to 20
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:POWer:SUM:PEP?')
		return Conversions.str_to_float(response)

	def get_value(self) -> float:
		"""SCPI: [SOURce<HW>]:AWGN:POWer:SUM \n
		Snippet: value: float = driver.source.awgn.power.sum.get_value() \n
		In Additive Noise and CW Interferer (SOUR:AWGN:MODE ADD|CW) modes, queries the overall power of the noise signal plus
		useful signal. \n
			:return: sum: float Range: -145 to 20
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:AWGN:POWer:SUM?')
		return Conversions.str_to_float(response)



from typing import List

from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class CatalogCls:
	"""Catalog commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("catalog", core, parent)

	def get_length(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:CATalog:LENGth \n
		Snippet: value: int = driver.source.bb.arbitrary.waveform.catalog.get_length() \n
		Reads out the files in the default directory and returns the number of waveform files in this directory. The default
		directory is set using command method RsSgt.MassMemory.currentDirectory. Only files with the file extension *.wv will be
		listed. \n
			:return: length: integer Number of waveform files in default directory Range: 0 to INT_MAX
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WAVeform:CATalog:LENGth?')
		return Conversions.str_to_int(response)

	def get_value(self) -> List[str]:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:CATalog \n
		Snippet: value: List[str] = driver.source.bb.arbitrary.waveform.catalog.get_value() \n
		Reads out the files in the default directory. The default directory is set using command method RsSgt.MassMemory.
		currentDirectory. When the names of the waveform files are returned they are separated by commas. Only files with the
		file extension *.wv will be listed. \n
			:return: catalog: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WAVeform:CATalog?')
		return Conversions.str_to_str_list(response)



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class InputPyCls:
	"""InputPy commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("inputPy", core, parent)

	def get_level(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:INPut:LEVel \n
		Snippet: value: float = driver.source.iq.doherty.inputPy.get_level() \n
		No command help available \n
			:return: level: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:DOHerty:INPut:LEVel?')
		return Conversions.str_to_float(response)

	def get_pep(self) -> float:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:INPut:PEP \n
		Snippet: value: float = driver.source.iq.doherty.inputPy.get_pep() \n
		No command help available \n
			:return: pep: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:DOHerty:INPut:PEP?')
		return Conversions.str_to_float(response)



"""Interface for SCPI communication logging."""
import socket
from enum import Enum
from datetime import datetime
import re
import string
from typing import List
from textwrap import wrap

from .Utilities import get_plural_string, shorten_string_middle, escape_nonprintable_chars, size_to_kb_mb_string, calculate_chunks_count
from .InstrumentErrors import RsInstrException
from .Conversions import list_to_csv_str, convert_ts_to_datetime, get_timedelta_string, get_timestamp_string, get_timedelta_fixed_string
from .GlobalData import GlobalData


class LoggingMode(Enum):
    """Determines the format of the logging message."""
    Off = 0  # Don't write messages to log
    On = 1  # Write message to log
    Errors = 2  # Only log errors. This is like 'Off', with the exception, that VisaIOErrors are logged.
    Default = 3  # Default mode


class LogEntry:
    """One entry in the log Defined by content, which has all the variables resolved except the START_TIME and the end_time."""

    _tab_var_re = re.compile(r'PAD_(LEFT|RIGHT)(\d+)\(%(START_TIME|END_TIME|DURATION|DEVICE_NAME|LOG_STRING_INFO|LOG_STRING)%\)')
    _var_re = re.compile(r'%(START_TIME|END_TIME|DURATION|DEVICE_NAME|LOG_STRING_INFO|LOG_STRING)%')

    def __init__(self, start_time: datetime or None or float, end_time: datetime or None or float, device_name: str, log_string_info: str, log_string: str, add_new_line: bool, error: bool, raw: bool, binary: bool):
        self._start_time: datetime = start_time
        self._end_time: datetime = end_time
        self._device_name: str = device_name
        self._log_string_info: str = log_string_info
        self._log_string: str = log_string
        self._raw: bool = raw
        self._binary: bool = binary
        self._timestamp_reference_time: datetime or None = None

        # Public properties
        self.add_new_line: bool = add_new_line
        self.error: bool = error

    @classmethod
    def as_raw_content(cls, content: str, add_new_line: bool) -> 'LogEntry':
        """Create the entry as raw"""
        return cls(start_time=None, end_time=None, device_name='', log_string_info='', log_string=content, add_new_line=add_new_line, error=False, raw=True, binary=False)

    @classmethod
    def as_raw_error_content(cls, content: str, add_new_line: bool) -> 'LogEntry':
        """Create the entry as raw"""
        return cls(start_time=None, end_time=None, device_name='', log_string_info='', log_string=content, add_new_line=add_new_line, error=True, raw=True, binary=False)

    @classmethod
    def as_info_entry(cls, start_time: datetime or None, end_time: datetime or None, device_name: str, log_string_info: str, log_string: str, add_new_line: bool) -> 'LogEntry':
        """Create the entry as info entry."""
        return cls(start_time=start_time, end_time=end_time, device_name=device_name, log_string_info=log_string_info, log_string=log_string, add_new_line=add_new_line, error=False, raw=False, binary=False)

    @classmethod
    def as_error_entry(cls, start_time: datetime or None, end_time: datetime or None, device_name: str, log_string_info: str, log_string: str, add_new_line: bool) -> 'LogEntry':
        """Create the entry as info entry."""
        return cls(start_time=start_time, end_time=end_time, device_name=device_name, log_string_info=log_string_info, log_string=log_string, add_new_line=add_new_line, error=True, raw=False, binary=False)

    @classmethod
    def as_info_bin_entry(cls, start_time: datetime or None, end_time: datetime or None, device_name: str, log_string_info: str, log_string: str, add_new_line: bool) -> 'LogEntry':
        """Create the entry as info entry."""
        return cls(start_time=start_time, end_time=end_time, device_name=device_name, log_string_info=log_string_info, log_string=log_string, add_new_line=add_new_line, error=False, raw=False, binary=True)

    def set_timestamp_reference_time(self, ref_time: datetime):
        """Sets reference time for the start time. None (default value) means the start time is absolute."""
        self._timestamp_reference_time = ref_time

    def get_resolved_content(self, template: str, encoding: str) -> str:
        """Returns the resolved content. For raw entry it means only the log_string."""
        if self._raw is True:
            return self._log_string

        result_str = self._replace_variables(template)
        # For non-binary data, trim the end and escape the non-printable chars.
        if self._binary is False:
            result_str = result_str.rstrip(': ')
            result_str = escape_nonprintable_chars(result_str, encoding)
        return result_str

    def _replace_variables(self, format_string: str) -> str:
        """Replaces all the variables in the entered template string with their actual values."""
        content = format_string
        while True:
            m = self._tab_var_re.search(content)
            if m:
                pos_left = m.group(1) == 'LEFT'
                spaces = int(m.group(2).strip())
                var_name = m.group(3).strip().strip('%')
                value = self._get_log_string_variable_values(var_name)
                value = value.rjust(spaces) if pos_left else value.ljust(spaces)
                content = content[:m.start()] + value + content[m.end():]
                continue
            m = self._var_re.search(content)
            if m:
                var_name = m.group(1)
                value = self._get_log_string_variable_values(var_name)
                if not value:
                    print(f'Value: {value} var_name "{var_name}" log_info: {self._log_string_info}')
                content = content[:m.start()] + value + content[m.end():]
                continue
            break
        return content

    def _get_log_string_variable_values(self, name: str) -> str:
        """Returns the required variable value.
        Recognised names: START_TIME, END_TIME, DURATION, DEVICE_NAME, LOG_STRING_INFO, LOG_STRING"""
        if name == 'START_TIME':
            if self._start_time is None:
                return ''
            if self._timestamp_reference_time is None:
                return get_timestamp_string(self._start_time)
            else:
                return get_timedelta_fixed_string(self._timestamp_reference_time, self._start_time)
        if name == 'END_TIME':
            if self._end_time is None:
                return ''
            return get_timestamp_string(self._end_time)
        if name == 'DURATION':
            if self._start_time is None or self._end_time is None:
                return ''
            return get_timedelta_string(self._start_time, self._end_time)
        if name == 'DEVICE_NAME':
            if self._device_name is None:
                return ''
            return self._device_name
        if name == 'LOG_STRING_INFO':
            if self._log_string_info is None:
                return ''
            return self._log_string_info
        if name == 'LOG_STRING':
            return self._log_string


class Segment:
    """Segment of logs"""

    def __init__(self):
        self.error_present = False
        self.entries: List[LogEntry] = []

    def add_to_segment(self, entry: LogEntry) -> None:
        """Adds an entry to the current segment."""
        self.entries.append(entry)
        if entry.error is True:
            self.error_present = True

    def empty(self) -> None:
        """Empties the list of entries for the current segment."""
        self.error_present = False
        self.entries = []

    def __del__(self):
        self.empty()


class CachedEntries:
    """Entries that are cached internally, in case no target is defined, but the logging is ON"""

    def __init__(self):
        self.entries: List[LogEntry] = []
        self.truncated_count = 0
        self.clear()

    def append(self, entry: LogEntry) -> None:
        """Appends one entry. If the list is longer than 1000 entries, the oldest one is deleted and truncated_count is increased by 1."""
        self.entries.append(entry)
        if len(self.entries) > 1000:
            self.entries.pop(0)
            self.truncated_count += 1
        return

    def clear(self) -> None:
        """Clears all the cached entries."""
        self.entries = []
        self.truncated_count = 0

    def __del__(self):
        self.clear()


class ScpiLogger:
    """Base class for SCPI logging"""

    def __init__(self, resource_name: str, encoding: str = 'charmap'):
        if resource_name is None:
            raise RsInstrException('resource_name cannot be None')
        self._orig_resource_name = resource_name
        self._global_mode: bool = False
        self._default_mode: LoggingMode = LoggingMode.Off
        self._mode: LoggingMode = self._default_mode
        self._log_target_local = None
        self._cached = CachedEntries()
        self._timestamp_reference_time_local: datetime or None = None
        self._format_string: str = ''
        self._line_divider: str = '\n'
        self._target_auto_flushing = True
        self.restore_format_string()
        self._log_to_console = False
        self._log_to_udp = False
        self._udp_port = 49200

        # Transients
        self._segment: Segment or None = None
        self._log_status_check_ok: bool = True
        self._socket = None

        # Printable chars set
        self._printable_chars = set(bytes(string.printable, 'ascii'))
        self._printable_chars.remove(10)
        self._printable_chars.remove(13)
        self._printable_chars.remove(32)
        self._printable_chars.remove(9)

        # Public properties
        self.device_name: str = resource_name
        """Use this property to change the resource name in the log from the default Resource Name (e.g. TCPIP::192.168.2.101::INSTR)
         to another name e.g. 'MySigGen1'."""
        self.abbreviated_max_len_ascii: int = 200
        """Defines the maximum length of one ASCII log entry. Default value is 200 characters."""
        self.abbreviated_max_len_list: int = 100
        """Defines the maximum length of one list entry. Default value is 100 elements."""
        self.abbreviated_max_len_bin: int = 2048
        """Defines the maximum length of one Binary log entry. Default value is 2048 bytes."""
        self.bin_line_block_size: int = 16
        """Defines number of bytes to display in one line. Default value is 16 bytes."""
        self.encoding = encoding
        """Defines encoding of the strings into bytes. Default is charmap."""
        self.allow_log_string_adjust = True
        """Specifies whether log string adjustment is allowed. Default is True."""

    def __str__(self):
        value = f"ScpiLogger for '{self.device_name}'"
        if self.mode != LoggingMode.Off:
            value += f' mode: {self._mode.name}'
        else:
            value += f' OFF'
        if self.log_to_console is False and self._log_to_udp is False and self.get_logging_target() is None:
            return value
        value += ', target: '
        if self.log_to_console:
            value += 'console + '
        if self.log_to_udp:
            value += 'udp + '
        if self._global_mode:
            value += 'global stream'
        elif self._log_target_local:
            value += 'stream'
        return value.strip('+ ')

    def start_new_segment(self) -> None:
        """Only relevant for if the log mode is LoggingMode.Errors.
        In this mode, all the logging is delayed until you call end_current_segment()
        Then, only if an error occurred, the logs are flushed to the output log.
        In case no error occurred, the logs are deleted."""
        if self._segment:
            self.end_current_segment()
            raise RsInstrException('Segment was not property finished before the new one was started.')
        # Logging to segment is only started if the logging mode is set to LoggingMode.Errors
        # That causes in mode ON all the entries appear immediately, rather than being delayed until the segment is ended
        self._segment = None
        if self._mode == LoggingMode.Errors:
            self._segment = Segment()

    def end_current_segment(self) -> None:
        """Only relevant for if the log mode is LoggingMode.Errors.
        Calling this method causes the logger to either flush all the segment logs to the output log or delete them if no error occurred.
        This causes only the errors to be logged, but also with the appropriate context, so you can troubleshoot more easily."""
        if self._segment:
            curr_segment = self._segment
            self._segment = None
            if self._mode == LoggingMode.Errors and curr_segment.error_present:
                # If error mode is on, and the segment contains at least one error entry, write the segment entries to the log
                for entry in curr_segment.entries:
                    self._write_to_log(entry)

    def set_format_string(self, value: str, line_divider: str = '\n') -> None:
        """Sets new format string and line divider.
        If you just want to set the line divider, set the format string value=None
        The original format string is: ``PAD_LEFT12(%START_TIME%) PAD_LEFT25(%DEVICE_NAME%) PAD_LEFT12(%DURATION%)  %LOG_STRING_INFO%: %LOG_STRING%`` """
        if value is not None:
            self._format_string = value
        self._line_divider = line_divider

    def restore_format_string(self) -> None:
        """Restores the original format string and the line divider to LF """
        self._format_string = 'PAD_LEFT12(%START_TIME%) PAD_LEFT30(%DEVICE_NAME%) PAD_LEFT12(%DURATION%)  %LOG_STRING_INFO%: %LOG_STRING%'
        self._line_divider = '\n'

    def clear_cached_entries(self) -> None:
        """Clears potential cached log entries.
        Cached log entries are generated when the Logging is ON, but no target has been defined yet."""
        self._cached.clear()

    def set_logging_target(self, target, console_log: bool or None = None, udp_log: bool or None = None) -> None:
        """Sets logging target - the target must implement write() and flush().
        You can optionally set the console and UDP logging ON or OFF.
        This method switches the logging target global OFF."""
        self._log_target_local = target
        self._global_mode = False
        if console_log is not None:
            self._log_to_console = console_log
        if udp_log is not None:
            self._log_to_udp = udp_log
        self._flush_cached_entries()

    def set_logging_target_global(self, console_log: bool or None = None, udp_log: bool or None = None) -> None:
        """Sets logging target to global. The global target must be defined. You can optionally set the console and UDP logging ON or OFF. """
        if GlobalData.get_logging_target() is None:
            raise RsInstrException(f"Cannot set the logging to global target, because the global target has not been defined. Device name: '{self.device_name}'")
        self._global_mode = True
        if console_log is not None:
            self._log_to_console = console_log
        if udp_log is not None:
            self._log_to_udp = udp_log
        self._flush_cached_entries()

    def get_logging_target(self):
        """Based on the global_mode, it returns the logging target: either the local or the global one."""
        return self._log_target_local if self._global_mode is False else GlobalData.get_logging_target()

    def get_relative_timestamp(self) -> datetime or None:
        """Based on the global_mode, it returns the relative timestamp: either the local or the global one."""
        return self._timestamp_reference_time_local if self._global_mode is False else GlobalData.get_logging_relative_timestamp()

    @property
    def target_auto_flushing(self) -> bool:
        """Returns status of the auto-flushing for the logging target."""
        return self._target_auto_flushing

    @target_auto_flushing.setter
    def target_auto_flushing(self, value: bool) -> None:
        """Sets auto-flushing for the logging target. If set to True (default value), after each entry the stream is flushed,
        This makes sure, that even if your e.g. file stream is not closed at the end, it contains all the entries."""
        self._target_auto_flushing = value

    @property
    def mode(self) -> LoggingMode:
        """Sets / returns the Logging mode.

        :Data Type: LoggingMode"""
        return self._mode

    @mode.setter
    def mode(self, value: LoggingMode) -> None:
        """Sets / returns the Logging mode."""
        if self._segment:
            raise RsInstrException(f"Can not change the logging mode when a log segment is active. End the segment with ScpiLogger.end_current_segment(). Device name: '{self.device_name}'")
        value = self._resolve_log_mode(value)
        if self.mode != LoggingMode.Off:
            # logging is ON. Check the internal log entries and flush them to the target
            self._flush_cached_entries()
        self._mode = value
        if self._mode == LoggingMode.Off and self.get_logging_target():
            # Logging was switched off, flush the entries on the target
            self.flush()

    @property
    def log_status_check_ok(self) -> bool:
        """Sets / returns the current status of status checking OK.
        If True (default), the log contains logging of the status checking 'Status check: OK'.
        If False, the 'Status check: OK' is skipped - the log is more compact.
        Errors will still be logged."""
        return self._log_status_check_ok

    @log_status_check_ok.setter
    def log_status_check_ok(self, value: bool) -> None:
        """Sets new logging of status checking OK.
        If True (default), the log contains logging of the status checking 'Status check: OK'.
        If False, the 'Status check: OK' is skipped - the log is more compact.
        Errors will still be logged."""
        self._log_status_check_ok = value
        self.info(datetime.now(), None, 'Logging of \'Status Check OK\'', 'ON' if value is True else 'OFF')

    @property
    def log_to_console(self) -> bool:
        """Returns logging to console status."""
        return self._log_to_console

    @log_to_console.setter
    def log_to_console(self, value: bool) -> None:
        """Sets logging to console. Default value is False."""
        self._log_to_console = value
        if self._log_to_console is True:
            self._flush_cached_entries()

    @property
    def log_to_udp(self) -> bool:
        """Returns logging to UDP status."""
        return self._log_to_udp

    @log_to_udp.setter
    def log_to_udp(self, value: bool) -> None:
        """Sets logging to UDP. Default value is False."""
        self._log_to_udp = value
        if self._log_to_udp is True:
            self._flush_cached_entries()

    @property
    def log_to_console_and_udp(self) -> bool:
        """Returns true, if both logging to UDP and console in are True."""
        return self.log_to_udp and self.log_to_console

    @log_to_console_and_udp.setter
    def log_to_console_and_udp(self, value: bool) -> None:
        """Sets logging to UDP and console in one call.
        Compare to calling two methods, this one flushes potential cached entries to both targets."""
        self._log_to_console = value
        self._log_to_udp = value
        if self._log_to_console is True:
            self._flush_cached_entries()

    @property
    def udp_port(self) -> int:
        """Returns udp logging port."""
        return self._udp_port

    @udp_port.setter
    def udp_port(self, value: int) -> None:
        """Sets UDP logging port. Default value is 49200."""
        self._udp_port = value

    def set_relative_timestamp(self, timestamp: datetime) -> None:
        """If set, the further timestamps will be relative to the entered time."""
        self._timestamp_reference_time_local = convert_ts_to_datetime(timestamp)

    def set_relative_timestamp_now(self) -> None:
        """Sets the relative timestamp to the current time."""
        self.set_relative_timestamp(datetime.now())

    def clear_relative_timestamp(self) -> None:
        """Clears the reference time, and the further logging continues with absolute times."""
        self._timestamp_reference_time_local = None

    def _target_exists(self):
        """Returns true, if the target either console or udp, or stream exist."""
        return self.get_logging_target() is not None or self.log_to_console is True or self._log_to_udp is True

    def _flush_cached_entries(self) -> bool:
        """Flushes internal log to the log target. Returns true if flushed."""
        if not self._target_exists():
            return False
        if self._cached.truncated_count > 0:
            self._write_to_log(LogEntry.as_raw_content(f'----- Missing {self._cached.truncated_count} oldest entries ----------', True))
        for entry in self._cached.entries:
            entry.set_timestamp_reference_time(self.get_relative_timestamp())
            self._write_to_log(entry)
        self.clear_cached_entries()
        return True

    def _resolve_log_mode(self, value: LoggingMode) -> LoggingMode:
        """Resolves the potential default mode."""
        if value == LoggingMode.Default:
            return self._default_mode
        else:
            return value

    @property
    def default_mode(self) -> LoggingMode:
        """Sets / returns the default logging mode.
        You can recall the default mode by calling the logger.mode = LoggingMode.Default

        :Data Type: LoggingMode
        """
        return self._default_mode

    @default_mode.setter
    def default_mode(self, value: LoggingMode) -> None:
        """Sets the default logging mode.
        You can recall the default mode by calling the logger.mode = LoggingMode.Default"""
        if value == LoggingMode.Default:
            raise ValueError('LoggingMode.Default can not be set here. Use a specific value.')
        self._default_mode = value

    def _write_to_log(self, entry: LogEntry) -> None:
        """Logs the provided string to the target and optionally to the stdout.
        Error signals that the entry is an error entry."""
        # Check if the segment is active and if so, write the log only to the segment.
        if self._segment:
            self._segment.add_to_segment(entry)
            return
        if not self._target_exists():
            # No target is defined yet, cache the entries internally for now
            self._cached.append(entry)
            return

        # only now, before writing to the log target, resolve the content.
        entry.set_timestamp_reference_time(self.get_relative_timestamp())
        content = entry.get_resolved_content(self._format_string, self.encoding)

        if self.log_to_console:
            print(content)
        if self.log_to_udp:
            self._send_to_udp(content, entry.error)
        target = self.get_logging_target()
        if target:
            new_line = self._line_divider if entry.add_new_line else ''
            try:
                target.write(content + new_line)
                if self._target_auto_flushing:
                    target.flush()
            except Exception as e:
                msg = f'Error logging to the stream. Message: \'{content}\'. Error: {e.args[0]}'
                raise RsInstrException(msg)
        return

    def _send_to_udp(self, content: str, error: bool):
        """Sends the log string to the defined udp port. Any socket exception is consumed."""
        prefix = '[e]' if error else '[i]'
        msg = bytes(prefix + content, "utf-8")
        try:
            if self._socket is None:
                self._socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            self._socket.sendto(msg, ("127.0.0.1", self._udp_port))
        finally:
            return

    def info_raw(self, log_entry: str, add_new_line: bool = True) -> None:
        """Method for logging the raw string without any formatting."""
        if self.mode == LoggingMode.Off:
            return
        if self._mode == LoggingMode.Errors:
            # For errors logging mode, log the info only if the segment is active.
            # That means, it might be logged eventually as a context when the segment ends with an error
            if not self._segment:
                return
        entry = LogEntry.as_raw_content(log_entry, add_new_line)
        self._write_to_log(entry)

    def _info_as_raw_entry(self, log_entry: LogEntry) -> None:
        """Method for logging the entry."""
        if self.mode == LoggingMode.Off:
            return
        if self._mode == LoggingMode.Errors:
            # For errors logging mode, log the info only if the segment is active.
            # That means, it might be logged eventually as a context when the segment ends with an error
            if not self._segment:
                return
        self._write_to_log(log_entry)

    def info(self, start_time: datetime or float or None, end_time: datetime or float or None, log_string_info: str, log_string: str) -> None:
        """Method for logging one info entry. For binary log_string, use the info_bin()"""
        if self.mode == LoggingMode.Off:
            return
        entry = self._compose_log_entry(start_time, end_time, log_string_info, log_string, self.abbreviated_max_len_ascii)
        entry.add_new_line = True
        self._info_as_raw_entry(entry)

    def info_bin(self, start_time: datetime or float or None, end_time: datetime or float or None, log_string_info: str, log_data: bytes) -> None:
        """Method for logging one info entry where the log_data is binary (bytes)."""
        if self.mode == LoggingMode.Off:
            return
        entry = self._compose_bin_log_entry(start_time, end_time, log_string_info, log_data)
        entry.add_new_line = True
        self._info_as_raw_entry(entry)

    def info_list(self, start_time: datetime or float or None, end_time: datetime or float or None, log_string_info: str, list_data: List) -> None:
        """Method for logging one info entry where the list_data is decimal List[]."""
        if self.mode == LoggingMode.Off:
            return
        delimiter = ', '
        if len(list_data) <= self.abbreviated_max_len_list:
            log_string = f'List size {len(list_data)}: {list_to_csv_str(list_data, delimiter=delimiter)}'
        else:
            chunk = self.abbreviated_max_len_list // 2
            log_string = f'List size {len(list_data)}, showing first and last {chunk} elements: '
            log_string += list_to_csv_str(list_data[:chunk], delimiter=delimiter)
            log_string += ' .... '
            log_string += list_to_csv_str(list_data[-chunk:], delimiter=delimiter)
        entry = self._compose_log_entry(start_time, end_time, log_string_info, log_string, max_log_string_len=None)
        entry.add_new_line = True
        self._info_as_raw_entry(entry)

    def error_raw(self, log_entry: str, add_new_line: bool = True) -> None:
        """Method for logging one error entry without any formatting. Setting the error flag to True."""
        if self.mode == LoggingMode.Off:
            return
        if self._segment:
            self._segment.error_present = True
        entry = LogEntry.as_raw_error_content(log_entry, add_new_line)
        self._write_to_log(entry)

    def _error_as_raw_entry(self, log_entry: LogEntry) -> None:
        """Method for logging one error entry."""
        if self.mode == LoggingMode.Off:
            return
        if self._segment:
            self._segment.error_present = True
        log_entry.error = True
        log_entry.add_new_line = True
        self._write_to_log(log_entry)

    def error(self, start_time: datetime or float or None, end_time: datetime or float or None, log_string_info: str, log_string: str) -> None:
        """Method for logging one error entry."""
        if self.mode == LoggingMode.Off:
            return
        entry = self._compose_log_entry(start_time, end_time, log_string_info, log_string, self.abbreviated_max_len_ascii)
        self._error_as_raw_entry(entry)

    def _adjust_log_strings(self, value: str) -> str:
        """Adjusts the log string to prevent repeating of the information in the fields."""
        # Prevent repeating of the device name:
        if self.allow_log_string_adjust and 'DEVICE_NAME' in self._format_string:
            value = value.replace(f"'{self._orig_resource_name}': ", '')
            value = value.replace(f"'{self._orig_resource_name}' ", '')
            value = value.replace(f"'{self._orig_resource_name}'", '')
            value = value.replace(self._orig_resource_name, '')
        return value

    def _adjust_for_repeated_log_string_info(self, log_string_info: str, log_string) -> str:
        # Prevent repeating of the log string info in the log_string
        if 'LOG_STRING_INFO' in self._format_string:
            log_string = log_string.replace(f'{log_string_info}: ', '')
            log_string = log_string.replace(f'{log_string_info} ', '')
            log_string = log_string.replace(f'{log_string_info}', '')
            log_string = log_string.replace(log_string_info, '')
        return log_string

    def _compose_log_entry(self, start_time: datetime or float, end_time: datetime or float, log_string_info: str, log_string: str, max_log_string_len: int = None) -> LogEntry:
        """Composes the log string with the format defined in the self._format_string"""
        log_string_info = self._adjust_log_strings(log_string_info)
        log_string = self._adjust_log_strings(log_string)
        log_string = self._adjust_for_repeated_log_string_info(log_string_info, log_string)
        orig_len = len(log_string)
        # Shorten the long log strings
        if max_log_string_len is not None:
            if orig_len > self.abbreviated_max_len_ascii:
                log_string = shorten_string_middle(log_string, max_log_string_len)
        entry = LogEntry(start_time, end_time, self.device_name, log_string_info, log_string, add_new_line=False, error=False, raw=False, binary=False)
        return entry

    def _compose_bin_log_entry(self, start_time: datetime or float, end_time: datetime or float, log_string_info: str, log_data: bytes) -> LogEntry:
        """Composes the binary log string with the format defined in the self._format_string"""
        log_string_info = self._adjust_log_strings(log_string_info)
        log_string_info = escape_nonprintable_chars(log_string_info, self.encoding)
        log_string = self._compose_hexdump(log_data, offset_left=20)
        entry = LogEntry(start_time, end_time, self.device_name, log_string_info, log_string, add_new_line=False, error=False, raw=False, binary=True)
        return entry

    def _compose_hexdump(self, value: str or bytes, offset_left: int) -> str:
        """Composes hexdump string from string or bytes.
        The hex dump is organised in the groups of 16 bytes per line."""
        if isinstance(value, str):
            value = bytes(value, self.encoding)
        size = len(value)
        line = f'{size_to_kb_mb_string(size, True)}'
        padding = ' ' * offset_left
        right_padding = self.bin_line_block_size * 3 - 1
        max_lines = calculate_chunks_count(self.abbreviated_max_len_bin, self.bin_line_block_size)
        lines_count = calculate_chunks_count(size, self.bin_line_block_size)
        skipping = False
        skip_start = 0
        skip_size = 0
        if lines_count > (max_lines + 1):
            # Skip the lines in the middle
            skip_start = max_lines // 2
            if skip_start == 0:
                skip_start = 1
            skip_size = lines_count - max_lines
            if skip_start > 0 and skip_size > 0:
                skipping = True
                line += f', showing {max_lines} blocks (1 block = {self.bin_line_block_size} bytes) out of {lines_count}, skipped {get_plural_string("block", skip_size)} in the middle'
                skip_start *= self.bin_line_block_size
                skip_size *= self.bin_line_block_size
        ix = 0
        end = 0
        line += self._line_divider
        while end < size:
            if skipping and ix >= skip_start:
                ix += skip_size
                line += padding + ('...' * self.bin_line_block_size) + self._line_divider
                skipping = False
            end = ix + self.bin_line_block_size
            if end >= size:
                end = size
            chunk = value[ix: end]
            cp = [chr(c) if c in self._printable_chars else '.' for c in chunk]
            line += padding + ' '.join(wrap(chunk.hex(), 2)).ljust(right_padding) + '    ' + ''.join(cp) + self._line_divider
            ix += self.bin_line_block_size
        return line

    def flush(self) -> None:
        """Flush all the entries."""
        target = self.get_logging_target()
        if target:
            try:
                # The file might be already closed.
                target.flush()
            except ValueError:
                pass

    def __del__(self):
        self.flush()

    def sync_from(self, source: 'ScpiLogger') -> None:
        """Synchronises this Logger with the source."""
        self._orig_resource_name = source._orig_resource_name
        self.mode = source.mode
        self.default_mode = source.default_mode
        self.log_status_check_ok = source.log_status_check_ok
        self.log_to_console = source.log_to_console
        self.log_to_udp = source.log_to_udp
        self.udp_port = source.udp_port
        self.set_format_string(source._format_string, source._line_divider)
        self._target_auto_flushing = source._target_auto_flushing

        self._log_target_local = source._log_target_local
        self._timestamp_reference_time_local = source._timestamp_reference_time_local
        if source._global_mode is True:
            self.set_logging_target_global()

        # Public properties
        self.device_name = source.device_name
        self.abbreviated_max_len_ascii = source.abbreviated_max_len_ascii
        self.abbreviated_max_len_list = source.abbreviated_max_len_list
        self.abbreviated_max_len_bin = source.abbreviated_max_len_bin
        self.bin_line_block_size = source.bin_line_block_size
        self.encoding = source.encoding
        self.allow_log_string_adjust = source.allow_log_string_adjust



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ...... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class DelayCls:
	"""Delay commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("delay", core, parent)

	# noinspection PyTypeChecker
	def get_unit(self) -> enums.TrigDelUnit:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:TRIGger:DELay:UNIT \n
		Snippet: value: enums.TrigDelUnit = driver.source.bb.arbitrary.trigger.delay.get_unit() \n
		Determines the units the trigger delay is expressed in. \n
			:return: trig_del_unit: SAMPle| TIME
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:TRIGger:DELay:UNIT?')
		return Conversions.str_to_scalar_enum(response, enums.TrigDelUnit)

	def set_unit(self, trig_del_unit: enums.TrigDelUnit) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:TRIGger:DELay:UNIT \n
		Snippet: driver.source.bb.arbitrary.trigger.delay.set_unit(trig_del_unit = enums.TrigDelUnit.SAMPle) \n
		Determines the units the trigger delay is expressed in. \n
			:param trig_del_unit: SAMPle| TIME
		"""
		param = Conversions.enum_scalar_to_str(trig_del_unit, enums.TrigDelUnit)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:TRIGger:DELay:UNIT {param}')



from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions
from ....... import repcap


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class PhaseCls:
	"""Phase commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("phase", core, parent)

	def set(self, phase: float, carrier=repcap.Carrier.Default) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CARRier<CH>:PHASe \n
		Snippet: driver.source.bb.arbitrary.mcarrier.carrier.phase.set(phase = 1.0, carrier = repcap.Carrier.Default) \n
		The command sets the start phase of the selected carrier. The phase settings are only valid if optimization of the crest
		factor is disabled (SOURce:BB:ARB:MCARrier:CFACtor:MODE OFF) . \n
			:param phase: float Range: 0 to 359.99, Unit: DEG
			:param carrier: optional repeated capability selector. Default value: Nr1 (settable in the interface 'Mcarrier')
		"""
		param = Conversions.decimal_value_to_str(phase)
		carrier_cmd_val = self._cmd_group.get_repcap_cmd_value(carrier, repcap.Carrier)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:CARRier{carrier_cmd_val}:PHASe {param}')

	def get(self, carrier=repcap.Carrier.Default) -> float:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CARRier<CH>:PHASe \n
		Snippet: value: float = driver.source.bb.arbitrary.mcarrier.carrier.phase.get(carrier = repcap.Carrier.Default) \n
		The command sets the start phase of the selected carrier. The phase settings are only valid if optimization of the crest
		factor is disabled (SOURce:BB:ARB:MCARrier:CFACtor:MODE OFF) . \n
			:param carrier: optional repeated capability selector. Default value: Nr1 (settable in the interface 'Mcarrier')
			:return: phase: float Range: 0 to 359.99, Unit: DEG"""
		carrier_cmd_val = self._cmd_group.get_repcap_cmd_value(carrier, repcap.Carrier)
		response = self._core.io.query_str(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:CARRier{carrier_cmd_val}:PHASe?')
		return Conversions.str_to_float(response)



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class McoderCls:
	"""Mcoder commands group definition. 3 total commands, 1 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("mcoder", core, parent)

	@property
	def arbitrary(self):
		"""arbitrary commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_arbitrary'):
			from .Arbitrary import ArbitraryCls
			self._arbitrary = ArbitraryCls(self._core, self._cmd_group)
		return self._arbitrary

	def get_value(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:PROGress:MCODer \n
		Snippet: value: int = driver.source.bb.progress.mcoder.get_value() \n
		Queries the status of an initiated process, like for example the calculation of a signal in accordance to a digital
		standard, or the calculation of a multi-carrier or multi-segment waveform file. \n
			:return: mcoder: integer Indicates the task progress in percent Range: 0 to 100
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:PROGress:MCODer?')
		return Conversions.str_to_int(response)

	def clone(self) -> 'McoderCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = McoderCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class BbandCls:
	"""Bband commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("bband", core, parent)

	def get_state(self) -> bool:
		"""SCPI: CALibration:IQModulator:BBANd:[STATe] \n
		Snippet: value: bool = driver.calibration.iqModulator.bband.get_state() \n
		No command help available \n
			:return: modulator: OFF| ON| 1| 0
		"""
		response = self._core.io.query_str('CALibration:IQModulator:BBANd:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, modulator: bool) -> None:
		"""SCPI: CALibration:IQModulator:BBANd:[STATe] \n
		Snippet: driver.calibration.iqModulator.bband.set_state(modulator = False) \n
		No command help available \n
			:param modulator: OFF| ON| 1| 0
		"""
		param = Conversions.bool_to_str(modulator)
		self._core.io.write(f'CALibration:IQModulator:BBANd:STATe {param}')



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class PrampCls:
	"""Pramp commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("pramp", core, parent)

	def get_state(self) -> bool:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:PRAMp:[STATe] \n
		Snippet: value: bool = driver.source.bb.arbitrary.pramp.get_state() \n
		If activated, the burst gate marker signal included in the ARB waveform file is used as marker signal for the pulse
		modulator. \n
			:return: arb_pram_state: 0| 1| OFF| ON
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:PRAMp:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, arb_pram_state: bool) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:PRAMp:[STATe] \n
		Snippet: driver.source.bb.arbitrary.pramp.set_state(arb_pram_state = False) \n
		If activated, the burst gate marker signal included in the ARB waveform file is used as marker signal for the pulse
		modulator. \n
			:param arb_pram_state: 0| 1| OFF| ON
		"""
		param = Conversions.bool_to_str(arb_pram_state)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:PRAMp:STATe {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ProfilingCls:
	"""Profiling commands group definition. 11 total commands, 5 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("profiling", core, parent)

	@property
	def hwAccess(self):
		"""hwAccess commands group. 0 Sub-classes, 3 commands."""
		if not hasattr(self, '_hwAccess'):
			from .HwAccess import HwAccessCls
			self._hwAccess = HwAccessCls(self._core, self._cmd_group)
		return self._hwAccess

	@property
	def logging(self):
		"""logging commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_logging'):
			from .Logging import LoggingCls
			self._logging = LoggingCls(self._core, self._cmd_group)
		return self._logging

	@property
	def module(self):
		"""module commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_module'):
			from .Module import ModuleCls
			self._module = ModuleCls(self._core, self._cmd_group)
		return self._module

	@property
	def tick(self):
		"""tick commands group. 1 Sub-classes, 1 commands."""
		if not hasattr(self, '_tick'):
			from .Tick import TickCls
			self._tick = TickCls(self._core, self._cmd_group)
		return self._tick

	@property
	def tpoint(self):
		"""tpoint commands group. 1 Sub-classes, 1 commands."""
		if not hasattr(self, '_tpoint'):
			from .Tpoint import TpointCls
			self._tpoint = TpointCls(self._core, self._cmd_group)
		return self._tpoint

	def get_state(self) -> bool:
		"""SCPI: SYSTem:PROFiling:STATe \n
		Snippet: value: bool = driver.system.profiling.get_state() \n
		No command help available \n
			:return: state: No help available
		"""
		response = self._core.io.query_str('SYSTem:PROFiling:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, state: bool) -> None:
		"""SCPI: SYSTem:PROFiling:STATe \n
		Snippet: driver.system.profiling.set_state(state = False) \n
		No command help available \n
			:param state: No help available
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'SYSTem:PROFiling:STATe {param}')

	def clone(self) -> 'ProfilingCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = ProfilingCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class TriggerCls:
	"""Trigger commands group definition. 2 total commands, 2 Subgroups, 0 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("trigger", core, parent)

	@property
	def impedance(self):
		"""impedance commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_impedance'):
			from .Impedance import ImpedanceCls
			self._impedance = ImpedanceCls(self._core, self._cmd_group)
		return self._impedance

	@property
	def slope(self):
		"""slope commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_slope'):
			from .Slope import SlopeCls
			self._slope = SlopeCls(self._core, self._cmd_group)
		return self._slope

	def clone(self) -> 'TriggerCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = TriggerCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ...... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class SeparatorCls:
	"""Separator commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("separator", core, parent)

	# noinspection PyTypeChecker
	def get_column(self) -> enums.DexchSepCol:
		"""SCPI: [SOURce<HW>]:CORRection:DEXChange:AFILe:SEParator:COLumn \n
		Snippet: value: enums.DexchSepCol = driver.source.correction.dexchange.afile.separator.get_column() \n
		Selects the separator between the frequency and level column of the ASCII table. \n
			:return: column: TABulator| SEMicolon| COMMa| SPACe
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:CORRection:DEXChange:AFILe:SEParator:COLumn?')
		return Conversions.str_to_scalar_enum(response, enums.DexchSepCol)

	def set_column(self, column: enums.DexchSepCol) -> None:
		"""SCPI: [SOURce<HW>]:CORRection:DEXChange:AFILe:SEParator:COLumn \n
		Snippet: driver.source.correction.dexchange.afile.separator.set_column(column = enums.DexchSepCol.COMMa) \n
		Selects the separator between the frequency and level column of the ASCII table. \n
			:param column: TABulator| SEMicolon| COMMa| SPACe
		"""
		param = Conversions.enum_scalar_to_str(column, enums.DexchSepCol)
		self._core.io.write(f'SOURce<HwInstance>:CORRection:DEXChange:AFILe:SEParator:COLumn {param}')

	# noinspection PyTypeChecker
	def get_decimal(self) -> enums.DexchSepDec:
		"""SCPI: [SOURce<HW>]:CORRection:DEXChange:AFILe:SEParator:DECimal \n
		Snippet: value: enums.DexchSepDec = driver.source.correction.dexchange.afile.separator.get_decimal() \n
		Selects the decimal separator used in the ASCII data between '.' (decimal point) and ',' (comma) with floating-point
		numerals. \n
			:return: decimal: DOT| COMMa
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:CORRection:DEXChange:AFILe:SEParator:DECimal?')
		return Conversions.str_to_scalar_enum(response, enums.DexchSepDec)

	def set_decimal(self, decimal: enums.DexchSepDec) -> None:
		"""SCPI: [SOURce<HW>]:CORRection:DEXChange:AFILe:SEParator:DECimal \n
		Snippet: driver.source.correction.dexchange.afile.separator.set_decimal(decimal = enums.DexchSepDec.COMMa) \n
		Selects the decimal separator used in the ASCII data between '.' (decimal point) and ',' (comma) with floating-point
		numerals. \n
			:param decimal: DOT| COMMa
		"""
		param = Conversions.enum_scalar_to_str(decimal, enums.DexchSepDec)
		self._core.io.write(f'SOURce<HwInstance>:CORRection:DEXChange:AFILe:SEParator:DECimal {param}')



from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions
from ....... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class CarrierCls:
	"""Carrier commands group definition. 10 total commands, 7 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("carrier", core, parent)

	@property
	def conflict(self):
		"""conflict commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_conflict'):
			from .Conflict import ConflictCls
			self._conflict = ConflictCls(self._core, self._cmd_group)
		return self._conflict

	@property
	def delay(self):
		"""delay commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_delay'):
			from .Delay import DelayCls
			self._delay = DelayCls(self._core, self._cmd_group)
		return self._delay

	@property
	def file(self):
		"""file commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_file'):
			from .File import FileCls
			self._file = FileCls(self._core, self._cmd_group)
		return self._file

	@property
	def frequency(self):
		"""frequency commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_frequency'):
			from .Frequency import FrequencyCls
			self._frequency = FrequencyCls(self._core, self._cmd_group)
		return self._frequency

	@property
	def phase(self):
		"""phase commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_phase'):
			from .Phase import PhaseCls
			self._phase = PhaseCls(self._core, self._cmd_group)
		return self._phase

	@property
	def power(self):
		"""power commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_power'):
			from .Power import PowerCls
			self._power = PowerCls(self._core, self._cmd_group)
		return self._power

	@property
	def state(self):
		"""state commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_state'):
			from .State import StateCls
			self._state = StateCls(self._core, self._cmd_group)
		return self._state

	def get_count(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CARRier:COUNt \n
		Snippet: value: int = driver.source.bb.arbitrary.mcarrier.carrier.get_count() \n
		Sets the number of carriers in the ARB multi carrier waveform. \n
			:return: count: integer Range: 1 to 512
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:CARRier:COUNt?')
		return Conversions.str_to_int(response)

	def set_count(self, count: int) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CARRier:COUNt \n
		Snippet: driver.source.bb.arbitrary.mcarrier.carrier.set_count(count = 1) \n
		Sets the number of carriers in the ARB multi carrier waveform. \n
			:param count: integer Range: 1 to 512
		"""
		param = Conversions.decimal_value_to_str(count)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:CARRier:COUNt {param}')

	# noinspection PyTypeChecker
	def get_mode(self) -> enums.ArbMultCarrSpacMode:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CARRier:MODE \n
		Snippet: value: enums.ArbMultCarrSpacMode = driver.source.bb.arbitrary.mcarrier.carrier.get_mode() \n
		The command sets the carrier frequency mode. \n
			:return: mode: EQUidistant| ARBitrary EQUidistant Sets an equidistant carrrier spacing. The carrier frequency in the carrier table is not configurable. ARBitrary Enables you to specify the carrier frequency in the carrier table. Carrier spacing is irrelevant.
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:CARRier:MODE?')
		return Conversions.str_to_scalar_enum(response, enums.ArbMultCarrSpacMode)

	def set_mode(self, mode: enums.ArbMultCarrSpacMode) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CARRier:MODE \n
		Snippet: driver.source.bb.arbitrary.mcarrier.carrier.set_mode(mode = enums.ArbMultCarrSpacMode.ARBitrary) \n
		The command sets the carrier frequency mode. \n
			:param mode: EQUidistant| ARBitrary EQUidistant Sets an equidistant carrrier spacing. The carrier frequency in the carrier table is not configurable. ARBitrary Enables you to specify the carrier frequency in the carrier table. Carrier spacing is irrelevant.
		"""
		param = Conversions.enum_scalar_to_str(mode, enums.ArbMultCarrSpacMode)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:CARRier:MODE {param}')

	def get_spacing(self) -> float:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CARRier:SPACing \n
		Snippet: value: float = driver.source.bb.arbitrary.mcarrier.carrier.get_spacing() \n
		The command sets the frequency spacing between adjacent carriers of the multi carrier waveform.
		The carriers are generated symmetrically around the RF carrier. The maximum carrier spacing is limited to Carrier spacing
		= Total baseband bandwidth /(Number of carriers - 1) . The total baseband bandwidth is 60, 120 or 160 MHz depending on
		the installed options. .
			INTRO_CMD_HELP: Note:In order to avoid wrap-around problems, the effective 'Carrier Spacing' might be slightly modified. The 'Carrier Spacing' is rounded in that way that the carrier closest to the center RF frequency shows no phase jump assuming that the carrier is unmodulated. \n
			- For odd number of carriers: RoundedCarrierSpacing=1/OutputSignalDuration* round(CarrierSpacing * OutputSignalDuration) ;
			- For even number of carriers: RoundedCarrierSpacing=2/OutputSignalDuration*round(0.5 *CarrierSpacing * OutputSignalDuration) ; \n
			:return: spacing: float Range: 0.0 to 120E6, Unit: Hz
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:CARRier:SPACing?')
		return Conversions.str_to_float(response)

	def set_spacing(self, spacing: float) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:CARRier:SPACing \n
		Snippet: driver.source.bb.arbitrary.mcarrier.carrier.set_spacing(spacing = 1.0) \n
		The command sets the frequency spacing between adjacent carriers of the multi carrier waveform.
		The carriers are generated symmetrically around the RF carrier. The maximum carrier spacing is limited to Carrier spacing
		= Total baseband bandwidth /(Number of carriers - 1) . The total baseband bandwidth is 60, 120 or 160 MHz depending on
		the installed options. .
			INTRO_CMD_HELP: Note:In order to avoid wrap-around problems, the effective 'Carrier Spacing' might be slightly modified. The 'Carrier Spacing' is rounded in that way that the carrier closest to the center RF frequency shows no phase jump assuming that the carrier is unmodulated. \n
			- For odd number of carriers: RoundedCarrierSpacing=1/OutputSignalDuration* round(CarrierSpacing * OutputSignalDuration) ;
			- For even number of carriers: RoundedCarrierSpacing=2/OutputSignalDuration*round(0.5 *CarrierSpacing * OutputSignalDuration) ; \n
			:param spacing: float Range: 0.0 to 120E6, Unit: Hz
		"""
		param = Conversions.decimal_value_to_str(spacing)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:CARRier:SPACing {param}')

	def clone(self) -> 'CarrierCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = CarrierCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from .....Internal.Utilities import trim_str_response
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class XmRadioCls:
	"""XmRadio commands group definition. 85 total commands, 5 Subgroups, 5 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("xmRadio", core, parent)

	@property
	def data(self):
		"""data commands group. 1 Sub-classes, 2 commands."""
		if not hasattr(self, '_data'):
			from .Data import DataCls
			self._data = DataCls(self._core, self._cmd_group)
		return self._data

	@property
	def satellite(self):
		"""satellite commands group. 4 Sub-classes, 1 commands."""
		if not hasattr(self, '_satellite'):
			from .Satellite import SatelliteCls
			self._satellite = SatelliteCls(self._core, self._cmd_group)
		return self._satellite

	@property
	def setting(self):
		"""setting commands group. 1 Sub-classes, 3 commands."""
		if not hasattr(self, '_setting'):
			from .Setting import SettingCls
			self._setting = SettingCls(self._core, self._cmd_group)
		return self._setting

	@property
	def terrestrial(self):
		"""terrestrial commands group. 5 Sub-classes, 1 commands."""
		if not hasattr(self, '_terrestrial'):
			from .Terrestrial import TerrestrialCls
			self._terrestrial = TerrestrialCls(self._core, self._cmd_group)
		return self._terrestrial

	@property
	def trigger(self):
		"""trigger commands group. 2 Sub-classes, 1 commands."""
		if not hasattr(self, '_trigger'):
			from .Trigger import TriggerCls
			self._trigger = TriggerCls(self._core, self._cmd_group)
		return self._trigger

	def get_fcounter(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:FCOunter \n
		Snippet: value: int = driver.source.bb.xmRadio.get_fcounter() \n
		No command help available \n
			:return: fcounter: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:XMRadio:FCOunter?')
		return Conversions.str_to_int(response)

	# noinspection PyTypeChecker
	def get_layer(self) -> enums.XmRadioPhysLayer:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:LAYer \n
		Snippet: value: enums.XmRadioPhysLayer = driver.source.bb.xmRadio.get_layer() \n
		No command help available \n
			:return: layer: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:XMRadio:LAYer?')
		return Conversions.str_to_scalar_enum(response, enums.XmRadioPhysLayer)

	def set_layer(self, layer: enums.XmRadioPhysLayer) -> None:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:LAYer \n
		Snippet: driver.source.bb.xmRadio.set_layer(layer = enums.XmRadioPhysLayer.SAT1A) \n
		No command help available \n
			:param layer: No help available
		"""
		param = Conversions.enum_scalar_to_str(layer, enums.XmRadioPhysLayer)
		self._core.io.write(f'SOURce<HwInstance>:BB:XMRadio:LAYer {param}')

	def preset(self) -> None:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:PRESet \n
		Snippet: driver.source.bb.xmRadio.preset() \n
		No command help available \n
		"""
		self._core.io.write(f'SOURce<HwInstance>:BB:XMRadio:PRESet')

	def preset_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:PRESet \n
		Snippet: driver.source.bb.xmRadio.preset_with_opc() \n
		No command help available \n
		Same as preset, but waits for the operation to complete before continuing further. Use the RsSgt.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'SOURce<HwInstance>:BB:XMRadio:PRESet', opc_timeout_ms)

	def get_state(self) -> bool:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:STATe \n
		Snippet: value: bool = driver.source.bb.xmRadio.get_state() \n
		No command help available \n
			:return: state: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:XMRadio:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, state: bool) -> None:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:STATe \n
		Snippet: driver.source.bb.xmRadio.set_state(state = False) \n
		No command help available \n
			:param state: No help available
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'SOURce<HwInstance>:BB:XMRadio:STATe {param}')

	def get_version(self) -> str:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:VERSion \n
		Snippet: value: str = driver.source.bb.xmRadio.get_version() \n
		No command help available \n
			:return: version: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:XMRadio:VERSion?')
		return trim_str_response(response)

	def clone(self) -> 'XmRadioCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = XmRadioCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from typing import List

from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class TpointCls:
	"""Tpoint commands group definition. 2 total commands, 1 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("tpoint", core, parent)

	@property
	def catalog(self):
		"""catalog commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_catalog'):
			from .Catalog import CatalogCls
			self._catalog = CatalogCls(self._core, self._cmd_group)
		return self._catalog

	def get_restart(self) -> List[str]:
		"""SCPI: SYSTem:PROFiling:TPOint:RESTart \n
		Snippet: value: List[str] = driver.system.profiling.tpoint.get_restart() \n
		No command help available \n
			:return: module_and_tp: No help available
		"""
		response = self._core.io.query_str('SYSTem:PROFiling:TPOint:RESTart?')
		return Conversions.str_to_str_list(response)

	def set_restart(self, module_and_tp: List[str]) -> None:
		"""SCPI: SYSTem:PROFiling:TPOint:RESTart \n
		Snippet: driver.system.profiling.tpoint.set_restart(module_and_tp = ['1', '2', '3']) \n
		No command help available \n
			:param module_and_tp: No help available
		"""
		param = Conversions.list_to_csv_quoted_str(module_and_tp)
		self._core.io.write(f'SYSTem:PROFiling:TPOint:RESTart {param}')

	def clone(self) -> 'TpointCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = TpointCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class LockoutCls:
	"""Lockout commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("lockout", core, parent)

	def set_state(self, state: bool) -> None:
		"""SCPI: TEST<HW>:REMote:LOCKout:[STATe] \n
		Snippet: driver.test.remote.lockout.set_state(state = False) \n
		No command help available \n
			:param state: No help available
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'TEST<HwInstance>:REMote:LOCKout:STATe {param}')



from typing import List

from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class DataCls:
	"""Data commands group definition. 4 total commands, 0 Subgroups, 4 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("data", core, parent)

	def get_catalog(self) -> List[str]:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:SHAPing:NORMalized:DATA:CATalog \n
		Snippet: value: List[str] = driver.source.iq.doherty.shaping.normalized.data.get_catalog() \n
		No command help available \n
			:return: catalog: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:DOHerty:SHAPing:NORMalized:DATA:CATalog?')
		return Conversions.str_to_str_list(response)

	def load(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:SHAPing:NORMalized:DATA:LOAD \n
		Snippet: driver.source.iq.doherty.shaping.normalized.data.load(filename = '1') \n
		No command help available \n
			:param filename: No help available
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:IQ:DOHerty:SHAPing:NORMalized:DATA:LOAD {param}')

	def set_store(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:SHAPing:NORMalized:DATA:STORe \n
		Snippet: driver.source.iq.doherty.shaping.normalized.data.set_store(filename = '1') \n
		No command help available \n
			:param filename: No help available
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:IQ:DOHerty:SHAPing:NORMalized:DATA:STORe {param}')

	def get_value(self) -> bytes:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:SHAPing:NORMalized:DATA \n
		Snippet: value: bytes = driver.source.iq.doherty.shaping.normalized.data.get_value() \n
		No command help available \n
			:return: data: No help available
		"""
		response = self._core.io.query_bin_block('SOURce<HwInstance>:IQ:DOHerty:SHAPing:NORMalized:DATA?')
		return response

	def set_value(self, data: bytes) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:SHAPing:NORMalized:DATA \n
		Snippet: driver.source.iq.doherty.shaping.normalized.data.set_value(data = b'ABCDEFGH') \n
		No command help available \n
			:param data: No help available
		"""
		self._core.io.write_bin_block('SOURce<HwInstance>:IQ:DOHerty:SHAPing:NORMalized:DATA ', data)



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class StateCls:
	"""State commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("state", core, parent)

	# noinspection PyTypeChecker
	def get_pon(self) -> enums.PowOutpPonMode:
		"""SCPI: OUTPut<HW>:[STATe]:PON \n
		Snippet: value: enums.PowOutpPonMode = driver.output.state.get_pon() \n
		Selects the state of the RF output when the instrument is switched on. \n
			:return: pon: OFF| UNCHanged OFF Deactivates the output when the instrument is switched on ([RF off]) . UNCHanged Restores the initial state of the RF output before the last turn off.
		"""
		response = self._core.io.query_str('OUTPut<HwInstance>:STATe:PON?')
		return Conversions.str_to_scalar_enum(response, enums.PowOutpPonMode)

	def set_pon(self, pon: enums.PowOutpPonMode) -> None:
		"""SCPI: OUTPut<HW>:[STATe]:PON \n
		Snippet: driver.output.state.set_pon(pon = enums.PowOutpPonMode.OFF) \n
		Selects the state of the RF output when the instrument is switched on. \n
			:param pon: OFF| UNCHanged OFF Deactivates the output when the instrument is switched on ([RF off]) . UNCHanged Restores the initial state of the RF output before the last turn off.
		"""
		param = Conversions.enum_scalar_to_str(pon, enums.PowOutpPonMode)
		self._core.io.write(f'OUTPut<HwInstance>:STATe:PON {param}')

	def get_value(self) -> bool:
		"""SCPI: OUTPut<HW>:[STATe] \n
		Snippet: value: bool = driver.output.state.get_value() \n
		Activates the RF output signal ([RF on/off]) . \n
			:return: state: 1| ON| 0| OFF
		"""
		response = self._core.io.query_str('OUTPut<HwInstance>:STATe?')
		return Conversions.str_to_bool(response)

	def set_value(self, state: bool) -> None:
		"""SCPI: OUTPut<HW>:[STATe] \n
		Snippet: driver.output.state.set_value(state = False) \n
		Activates the RF output signal ([RF on/off]) . \n
			:param state: 1| ON| 0| OFF
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'OUTPut<HwInstance>:STATe {param}')



from ..Internal.Core import Core
from ..Internal.CommandsGroup import CommandsGroup
from ..Internal import Conversions
from .. import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class UnitCls:
	"""Unit commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("unit", core, parent)

	# noinspection PyTypeChecker
	def get_angle(self) -> enums.UnitAngle:
		"""SCPI: UNIT:ANGLe \n
		Snippet: value: enums.UnitAngle = driver.unit.get_angle() \n
		Sets the default angle unit for remote control. Does not influence the manual control parameter units and the display. \n
			:return: angle: DEGRee| RADian
		"""
		response = self._core.io.query_str('UNIT:ANGLe?')
		return Conversions.str_to_scalar_enum(response, enums.UnitAngle)

	def set_angle(self, angle: enums.UnitAngle) -> None:
		"""SCPI: UNIT:ANGLe \n
		Snippet: driver.unit.set_angle(angle = enums.UnitAngle.DEGree) \n
		Sets the default angle unit for remote control. Does not influence the manual control parameter units and the display. \n
			:param angle: DEGRee| RADian
		"""
		param = Conversions.enum_scalar_to_str(angle, enums.UnitAngle)
		self._core.io.write(f'UNIT:ANGLe {param}')

	# noinspection PyTypeChecker
	def get_power(self) -> enums.UnitPower:
		"""SCPI: UNIT:POWer \n
		Snippet: value: enums.UnitPower = driver.unit.get_power() \n
		Defines the default unit for power parameters. This setting affects the GUI, as well as all remote control commands that
		determine power values. \n
			:return: power: V| DBUV| DBM
		"""
		response = self._core.io.query_str('UNIT:POWer?')
		return Conversions.str_to_scalar_enum(response, enums.UnitPower)

	def set_power(self, power: enums.UnitPower) -> None:
		"""SCPI: UNIT:POWer \n
		Snippet: driver.unit.set_power(power = enums.UnitPower.DBM) \n
		Defines the default unit for power parameters. This setting affects the GUI, as well as all remote control commands that
		determine power values. \n
			:param power: V| DBUV| DBM
		"""
		param = Conversions.enum_scalar_to_str(power, enums.UnitPower)
		self._core.io.write(f'UNIT:POWer {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class InfoCls:
	"""Info commands group definition. 8 total commands, 3 Subgroups, 0 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("info", core, parent)

	@property
	def ecount(self):
		"""ecount commands group. 3 Sub-classes, 1 commands."""
		if not hasattr(self, '_ecount'):
			from .Ecount import EcountCls
			self._ecount = EcountCls(self._core, self._cmd_group)
		return self._ecount

	@property
	def otime(self):
		"""otime commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_otime'):
			from .Otime import OtimeCls
			self._otime = OtimeCls(self._core, self._cmd_group)
		return self._otime

	@property
	def poCount(self):
		"""poCount commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_poCount'):
			from .PoCount import PoCountCls
			self._poCount = PoCountCls(self._core, self._cmd_group)
		return self._poCount

	def clone(self) -> 'InfoCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = InfoCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions
from ....... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class NextCls:
	"""Next commands group definition. 3 total commands, 1 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("next", core, parent)

	@property
	def execute(self):
		"""execute commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_execute'):
			from .Execute import ExecuteCls
			self._execute = ExecuteCls(self._core, self._cmd_group)
		return self._execute

	# noinspection PyTypeChecker
	def get_source(self) -> enums.ArbSegmNextSourIntNext:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:NEXT:SOURce \n
		Snippet: value: enums.ArbSegmNextSourIntNext = driver.source.bb.arbitrary.wsegment.next.get_source() \n
		Selects the next segment source, i.e. determines whether the next segment is switched upon receiving of an external
		trigger signal or upon executing of software trigger . \n
			:return: source: INTernal| NEXT INTernal Switch over to any segment is triggered manually by means of the command BB:ARB:WSEG:NEXT:EXEC. NEXT The trigger event for switching to the next segment is input on the NEXT connector.
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WSEGment:NEXT:SOURce?')
		return Conversions.str_to_scalar_enum(response, enums.ArbSegmNextSourIntNext)

	def set_source(self, source: enums.ArbSegmNextSourIntNext) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:NEXT:SOURce \n
		Snippet: driver.source.bb.arbitrary.wsegment.next.set_source(source = enums.ArbSegmNextSourIntNext.INTernal) \n
		Selects the next segment source, i.e. determines whether the next segment is switched upon receiving of an external
		trigger signal or upon executing of software trigger . \n
			:param source: INTernal| NEXT INTernal Switch over to any segment is triggered manually by means of the command BB:ARB:WSEG:NEXT:EXEC. NEXT The trigger event for switching to the next segment is input on the NEXT connector.
		"""
		param = Conversions.enum_scalar_to_str(source, enums.ArbSegmNextSourIntNext)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:NEXT:SOURce {param}')

	def get_value(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:NEXT \n
		Snippet: value: int = driver.source.bb.arbitrary.wsegment.next.get_value() \n
		Selects the segment to be output. \n
			:return: next_py: float Range: 0 to 1023
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WSEGment:NEXT?')
		return Conversions.str_to_int(response)

	def set_value(self, next_py: int) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:NEXT \n
		Snippet: driver.source.bb.arbitrary.wsegment.next.set_value(next_py = 1) \n
		Selects the segment to be output. \n
			:param next_py: float Range: 0 to 1023
		"""
		param = Conversions.decimal_value_to_str(next_py)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:NEXT {param}')

	def clone(self) -> 'NextCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = NextCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class InputPyCls:
	"""InputPy commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("inputPy", core, parent)

	def get_frequency(self) -> float:
		"""SCPI: CLOCk:INPut:FREQuency \n
		Snippet: value: float = driver.clock.inputPy.get_frequency() \n
		Queries the measured frequency of the external clock signal. An external clock reference must be supplied at the
		[USER1/2] input. \n
			:return: frequency: float Range: 0 to max
		"""
		response = self._core.io.query_str('CLOCk:INPut:FREQuency?')
		return Conversions.str_to_float(response)

	# noinspection PyTypeChecker
	def get_slope(self) -> enums.SlopeType:
		"""SCPI: CLOCk:INPut:SLOPe \n
		Snippet: value: enums.SlopeType = driver.clock.inputPy.get_slope() \n
		The command sets the active slope of an externally applied clock signal at the [USER 1/2] connector. \n
			:return: slope: NEGative| POSitive
		"""
		response = self._core.io.query_str('CLOCk:INPut:SLOPe?')
		return Conversions.str_to_scalar_enum(response, enums.SlopeType)

	def set_slope(self, slope: enums.SlopeType) -> None:
		"""SCPI: CLOCk:INPut:SLOPe \n
		Snippet: driver.clock.inputPy.set_slope(slope = enums.SlopeType.NEGative) \n
		The command sets the active slope of an externally applied clock signal at the [USER 1/2] connector. \n
			:param slope: NEGative| POSitive
		"""
		param = Conversions.enum_scalar_to_str(slope, enums.SlopeType)
		self._core.io.write(f'CLOCk:INPut:SLOPe {param}')



from typing import List

from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions
from .......Internal.Types import DataType
from .......Internal.StructBase import StructBase
from .......Internal.ArgStruct import ArgStruct
from .......Internal.ArgSingleList import ArgSingleList
from .......Internal.ArgSingle import ArgSingle


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class CoefficientsCls:
	"""Coefficients commands group definition. 4 total commands, 0 Subgroups, 4 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("coefficients", core, parent)

	def set(self, ipart_0: List[float], j_0: float, i_1: float, j_1: float) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:SHAPing:POLYnomial:COEFficients \n
		Snippet: driver.source.iq.doherty.shaping.polynomial.coefficients.set(ipart_0 = [1.1, 2.2, 3.3], j_0 = 1.0, i_1 = 1.0, j_1 = 1.0) \n
		No command help available \n
			:param ipart_0: No help available
			:param j_0: No help available
			:param i_1: No help available
			:param j_1: No help available
		"""
		param = ArgSingleList().compose_cmd_string(ArgSingle.as_open_list('ipart_0', ipart_0, DataType.FloatList, None), ArgSingle('j_0', j_0, DataType.Float), ArgSingle('i_1', i_1, DataType.Float), ArgSingle('j_1', j_1, DataType.Float))
		self._core.io.write(f'SOURce<HwInstance>:IQ:DOHerty:SHAPing:POLYnomial:COEFficients {param}'.rstrip())

	# noinspection PyTypeChecker
	class CoefficientsStruct(StructBase):
		"""Response structure. Fields: \n
			- Ipart_0: List[float]: No parameter help available
			- J_0: float: No parameter help available
			- I_1: float: No parameter help available
			- J_1: float: No parameter help available"""
		__meta_args_list = [
			ArgStruct('Ipart_0', DataType.FloatList, None, False, True, 1),
			ArgStruct.scalar_float('J_0'),
			ArgStruct.scalar_float('I_1'),
			ArgStruct.scalar_float('J_1')]

		def __init__(self):
			StructBase.__init__(self, self)
			self.Ipart_0: List[float] = None
			self.J_0: float = None
			self.I_1: float = None
			self.J_1: float = None

	def get(self) -> CoefficientsStruct:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:SHAPing:POLYnomial:COEFficients \n
		Snippet: value: CoefficientsStruct = driver.source.iq.doherty.shaping.polynomial.coefficients.get() \n
		No command help available \n
			:return: structure: for return value, see the help for CoefficientsStruct structure arguments."""
		return self._core.io.query_struct(f'SOURce<HwInstance>:IQ:DOHerty:SHAPing:POLYnomial:COEFficients?', self.__class__.CoefficientsStruct())

	def get_catalog(self) -> List[str]:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:SHAPing:POLYnomial:COEFficients:CATalog \n
		Snippet: value: List[str] = driver.source.iq.doherty.shaping.polynomial.coefficients.get_catalog() \n
		No command help available \n
			:return: catalog: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:IQ:DOHerty:SHAPing:POLYnomial:COEFficients:CATalog?')
		return Conversions.str_to_str_list(response)

	def load(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:SHAPing:POLYnomial:COEFficients:LOAD \n
		Snippet: driver.source.iq.doherty.shaping.polynomial.coefficients.load(filename = '1') \n
		No command help available \n
			:param filename: No help available
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:IQ:DOHerty:SHAPing:POLYnomial:COEFficients:LOAD {param}')

	def set_store(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:IQ:DOHerty:SHAPing:POLYnomial:COEFficients:STORe \n
		Snippet: driver.source.iq.doherty.shaping.polynomial.coefficients.set_store(filename = '1') \n
		No command help available \n
			:param filename: No help available
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:IQ:DOHerty:SHAPing:POLYnomial:COEFficients:STORe {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ErrorCls:
	"""Error commands group definition. 7 total commands, 2 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("error", core, parent)

	@property
	def code(self):
		"""code commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_code'):
			from .Code import CodeCls
			self._code = CodeCls(self._core, self._cmd_group)
		return self._code

	@property
	def history(self):
		"""history commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_history'):
			from .History import HistoryCls
			self._history = HistoryCls(self._core, self._cmd_group)
		return self._history

	def get_all(self) -> str:
		"""SCPI: SYSTem:ERRor:ALL \n
		Snippet: value: str = driver.system.error.get_all() \n
		Queries the error/event queue for all unread items and removes them from the queue. \n
			:return: all_py: string Error/event_number,'Error/event_description[;Device-dependent info]' A comma separated list of error number and a short description of the error in FIFO order. If the queue is empty, the response is 0,'No error' Positive error numbers are instrument-dependent. Negative error numbers are reserved by the SCPI standard. Volatile errors are reported once, at the time they appear. Identical errors are reported repeatedly only if the original error has already been retrieved from (and hence not any more present in) the error queue.
		"""
		response = self._core.io.query_str('SYSTem:ERRor:ALL?')
		return trim_str_response(response)

	def get_count(self) -> str:
		"""SCPI: SYSTem:ERRor:COUNt \n
		Snippet: value: str = driver.system.error.get_count() \n
		Queries the number of entries in the error queue. \n
			:return: count: integer 0 The error queue is empty.
		"""
		response = self._core.io.query_str('SYSTem:ERRor:COUNt?')
		return trim_str_response(response)

	def get_static(self) -> str:
		"""SCPI: SYSTem:ERRor:STATic \n
		Snippet: value: str = driver.system.error.get_static() \n
		No command help available \n
			:return: static_errors: No help available
		"""
		response = self._core.io.query_str('SYSTem:ERRor:STATic?')
		return trim_str_response(response)

	def clone(self) -> 'ErrorCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = ErrorCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



"""See the class docstring."""

from enum import Enum
from typing import List
from .Core import Core
from .RepeatedCapability import RepeatedCapability as RepCap
from .InstrumentErrors import DriverValueError


class CommandsGroup:
	"""Contains methods dealing with RepCaps and Group object cloning"""

	def __init__(self, group_name: str, core: Core, parent: 'CommandsGroup'):
		"""Constructor with header name, group property name and the start value"""

		self.group_name = group_name
		self._core = core
		self.parent = parent
		self.io = core.io
		self.existing_children = []
		self.rep_cap: RepCap or None = None
		self.multi_repcap_types: str = ''
		if parent:
			parent.existing_children.append(self)

	def __str__(self):
		"""String representation of the CommandsGroup"""
		out = f"SCPI Commands Group {self.group_name}"
		if self.has_repcap():
			out += f', RepCap {self.rep_cap.name} = {self.rep_cap.get_enum_value()}'
		return out

	def has_repcap(self) -> bool:
		"""Returns True, if the group has a RepCap.
		Returns False for a group with MultiRepCaps"""
		return self.rep_cap is not None

	def has_multi_repcaps(self) -> bool:
		"""Returns True for a group with MultiRepCaps"""
		return self.multi_repcap_types != ''

	def add_existing_child(self, child: 'CommandsGroup') -> None:
		"""Adds the child to the parent's list of created children.
		This is used when the group is cloned, where the whole existing tree of groups have to be recreated"""
		self.existing_children.append(child)

	def set_repcap_enum_value(self, enum_value: Enum or int) -> None:
		"""Sets RepCap value as enum or integer
		Default is not allowed here."""
		try:
			self.rep_cap.set_enum_value(enum_value)
		except ValueError:
			raise DriverValueError(self.io.resource_name, f"Commands group RepCap value '{self.rep_cap.name}.Default' cannot be set. Please select a concrete value.")

	def get_repcap_enum_value(self) -> Enum:
		"""Returns RepCap value as enum"""
		return self.rep_cap.get_enum_value()

	def get_repcap_cmd_value(self, enum_value: Enum or int, enum_type) -> str:
		"""Returns the current string of RepCapCmdValue for the entered RepCapEnumName
		The enum_value can be a repcap of the current CommandsGroup or any of their parents"""
		# Use the static functions of the RepeatedCapability to get the non-default value
		# It is faster, since there is no need to use the RepCap instance
		if not RepCap.clsm_is_default_value(enum_value, enum_type):
			return RepCap.clsm_get_cmd_string_value(enum_value, enum_type)
		# Default value - get it from the group or the parent groups
		group = self
		while group is not None:
			if group.has_repcap():
				if group.rep_cap.matches_type(enum_type):
					return group.rep_cap.get_cmd_string_value()
			group = group.parent

		# repCapEnumName not found in single repcaps. Check the multiRepCaps
		group = self
		while group is not None:
			if group.has_multi_repcaps():
				if str(enum_type.__name__) in group.multi_repcap_types.split(','):
					# Found in the multiple repcaps, create more fitting exception message
					raise DriverValueError(
						self.io.resource_name,
						f"You can not use the RepCap value '{enum_value}', "
						f"because its real value is not defined in any of the parent command groups. Please select a concrete value.")
			group = group.parent

		# repCapEnumName not found, create exception message
		group = self
		groups = []
		while group is not None:
			item = group.group_name
			if group.has_repcap():
				item += f' => {group.rep_cap.name}'
			groups.insert(0, item)
			group = group.parent

		groups_chain = str.join("\n", groups)
		raise DriverValueError(
			self.io.resource_name,
			f"Error replacing RepCaps in the SCPI command:"
			f"RepCap '{enum_type}' not found in the group chain:\n{groups_chain}")

	def get_owners_chain(self, stop: 'CommandsGroup' = None) -> List['CommandsGroup']:
		"""Returns the owners chain including itself up to the entered point or up to the root by default"""
		chain = []
		group = self
		while group != stop:
			chain.append(group)
			group = group.parent
		return chain

	def get_self_and_desc_existing_children(self) -> List['CommandsGroup']:
		"""Get all the existing descendant groups recursively"""
		all_descendants = [self]
		for x in self.existing_children:
			all_descendants.extend(x.get_self_and_desc_existing_children())
		return all_descendants

	def synchronize_repcaps(self, new_group) -> None:
		"""Clones the existing group repeated capabilities to the new one.
		Because of the lazy group properties, the group clones are created by accessing the repcaps in them"""
		all_existing = filter(lambda grp: grp.has_repcap(), self.get_self_and_desc_existing_children())
		for x in all_existing:
			chain = x.get_owners_chain(self)
			chain.reverse()
			group = new_group
			for item in chain:
				group = getattr(group, item.group_name)
			fnc = getattr(group, x.rep_cap.method_set_name)
			fnc(x.rep_cap.get_enum_value())

	def restore_repcaps(self) -> None:
		"""Sets RepCaps of the Group and its children groups to their initial values"""
		all_existing = filter(lambda grp: grp.has_repcap(), self.get_self_and_desc_existing_children())
		for x in all_existing:
			x.rep_cap.set_to_start_value()



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class FpresetCls:
	"""Fpreset commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("fpreset", core, parent)

	def set(self) -> None:
		"""SCPI: SYSTem:FPReset \n
		Snippet: driver.system.fpreset.set() \n
		Triggers an instrument reset to the original state of delivery. \n
		"""
		self._core.io.write(f'SYSTem:FPReset')

	def set_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: SYSTem:FPReset \n
		Snippet: driver.system.fpreset.set_with_opc() \n
		Triggers an instrument reset to the original state of delivery. \n
		Same as set, but waits for the operation to complete before continuing further. Use the RsSgt.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'SYSTem:FPReset', opc_timeout_ms)



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class StepCls:
	"""Step commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("step", core, parent)

	# noinspection PyTypeChecker
	def get_mode(self) -> enums.FreqStepMode:
		"""SCPI: [SOURce<HW>]:FREQuency:[FIXed]:STEP:MODE \n
		Snippet: value: enums.FreqStepMode = driver.source.frequency.fixed.step.get_mode() \n
		No command help available \n
			:return: freq_step_mode: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:FREQuency:FIXed:STEP:MODE?')
		return Conversions.str_to_scalar_enum(response, enums.FreqStepMode)

	def set_mode(self, freq_step_mode: enums.FreqStepMode) -> None:
		"""SCPI: [SOURce<HW>]:FREQuency:[FIXed]:STEP:MODE \n
		Snippet: driver.source.frequency.fixed.step.set_mode(freq_step_mode = enums.FreqStepMode.DECimal) \n
		No command help available \n
			:param freq_step_mode: No help available
		"""
		param = Conversions.enum_scalar_to_str(freq_step_mode, enums.FreqStepMode)
		self._core.io.write(f'SOURce<HwInstance>:FREQuency:FIXed:STEP:MODE {param}')

	def get_increment(self) -> float:
		"""SCPI: [SOURce<HW>]:FREQuency:[FIXed]:STEP:[INCRement] \n
		Snippet: value: float = driver.source.frequency.fixed.step.get_increment() \n
		No command help available \n
			:return: freq_step: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:FREQuency:FIXed:STEP:INCRement?')
		return Conversions.str_to_float(response)

	def set_increment(self, freq_step: float) -> None:
		"""SCPI: [SOURce<HW>]:FREQuency:[FIXed]:STEP:[INCRement] \n
		Snippet: driver.source.frequency.fixed.step.set_increment(freq_step = 1.0) \n
		No command help available \n
			:param freq_step: No help available
		"""
		param = Conversions.decimal_value_to_str(freq_step)
		self._core.io.write(f'SOURce<HwInstance>:FREQuency:FIXed:STEP:INCRement {param}')



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class TestCls:
	"""Test commands group definition. 17 total commands, 9 Subgroups, 3 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("test", core, parent)

	@property
	def all(self):
		"""all commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_all'):
			from .All import AllCls
			self._all = AllCls(self._core, self._cmd_group)
		return self._all

	@property
	def bb(self):
		"""bb commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_bb'):
			from .Bb import BbCls
			self._bb = BbCls(self._core, self._cmd_group)
		return self._bb

	@property
	def bbin(self):
		"""bbin commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_bbin'):
			from .Bbin import BbinCls
			self._bbin = BbinCls(self._core, self._cmd_group)
		return self._bbin

	@property
	def connector(self):
		"""connector commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_connector'):
			from .Connector import ConnectorCls
			self._connector = ConnectorCls(self._core, self._cmd_group)
		return self._connector

	@property
	def keyboard(self):
		"""keyboard commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_keyboard'):
			from .Keyboard import KeyboardCls
			self._keyboard = KeyboardCls(self._core, self._cmd_group)
		return self._keyboard

	@property
	def remote(self):
		"""remote commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_remote'):
			from .Remote import RemoteCls
			self._remote = RemoteCls(self._core, self._cmd_group)
		return self._remote

	@property
	def res(self):
		"""res commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_res'):
			from .Res import ResCls
			self._res = ResCls(self._core, self._cmd_group)
		return self._res

	@property
	def serror(self):
		"""serror commands group. 1 Sub-classes, 1 commands."""
		if not hasattr(self, '_serror'):
			from .Serror import SerrorCls
			self._serror = SerrorCls(self._core, self._cmd_group)
		return self._serror

	@property
	def sw(self):
		"""sw commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_sw'):
			from .Sw import SwCls
			self._sw = SwCls(self._core, self._cmd_group)
		return self._sw

	# noinspection PyTypeChecker
	def get_eiq_mode(self) -> enums.TestExtIqMode:
		"""SCPI: TEST:EIQMode \n
		Snippet: value: enums.TestExtIqMode = driver.test.get_eiq_mode() \n
		Triggers a connection test for testing the active external IQ devices. \n
			:return: eiq_mode: IQIN| IQOut
		"""
		response = self._core.io.query_str('TEST:EIQMode?')
		return Conversions.str_to_scalar_enum(response, enums.TestExtIqMode)

	def set_eiq_mode(self, eiq_mode: enums.TestExtIqMode) -> None:
		"""SCPI: TEST:EIQMode \n
		Snippet: driver.test.set_eiq_mode(eiq_mode = enums.TestExtIqMode.IQIN) \n
		Triggers a connection test for testing the active external IQ devices. \n
			:param eiq_mode: IQIN| IQOut
		"""
		param = Conversions.enum_scalar_to_str(eiq_mode, enums.TestExtIqMode)
		self._core.io.write(f'TEST:EIQMode {param}')

	def set_nrp_trigger(self, nrp_trigger: bool) -> None:
		"""SCPI: TEST:NRPTrigger \n
		Snippet: driver.test.set_nrp_trigger(nrp_trigger = False) \n
		No command help available \n
			:param nrp_trigger: No help available
		"""
		param = Conversions.bool_to_str(nrp_trigger)
		self._core.io.write(f'TEST:NRPTrigger {param}')

	def preset(self) -> None:
		"""SCPI: TEST:PRESet \n
		Snippet: driver.test.preset() \n
		No command help available \n
		"""
		self._core.io.write(f'TEST:PRESet')

	def preset_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: TEST:PRESet \n
		Snippet: driver.test.preset_with_opc() \n
		No command help available \n
		Same as preset, but waits for the operation to complete before continuing further. Use the RsSgt.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'TEST:PRESet', opc_timeout_ms)

	def clone(self) -> 'TestCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = TestCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ...Internal.Core import Core
from ...Internal.CommandsGroup import CommandsGroup
from ...Internal import Conversions
from ... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class IdentificationCls:
	"""Identification commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("identification", core, parent)

	def preset(self) -> None:
		"""SCPI: SYSTem:IDENtification:PRESet \n
		Snippet: driver.system.identification.preset() \n
		No command help available \n
		"""
		self._core.io.write(f'SYSTem:IDENtification:PRESet')

	def preset_with_opc(self, opc_timeout_ms: int = -1) -> None:
		"""SCPI: SYSTem:IDENtification:PRESet \n
		Snippet: driver.system.identification.preset_with_opc() \n
		No command help available \n
		Same as preset, but waits for the operation to complete before continuing further. Use the RsSgt.utilities.opc_timeout_set() to set the timeout value. \n
			:param opc_timeout_ms: Maximum time to wait in milliseconds, valid only for this call."""
		self._core.io.write_with_opc(f'SYSTem:IDENtification:PRESet', opc_timeout_ms)

	# noinspection PyTypeChecker
	def get_value(self) -> enums.IecDevId:
		"""SCPI: SYSTem:IDENtification \n
		Snippet: value: enums.IecDevId = driver.system.identification.get_value() \n
		Selects the mode the instrument identification is performed. \n
			:return: identification: AUTO| USER AUTO The *IDN string and the *OPT string are set automatically. USER Enables the selection of user definable *IDN and *OPT strings.
		"""
		response = self._core.io.query_str('SYSTem:IDENtification?')
		return Conversions.str_to_scalar_enum(response, enums.IecDevId)

	def set_value(self, identification: enums.IecDevId) -> None:
		"""SCPI: SYSTem:IDENtification \n
		Snippet: driver.system.identification.set_value(identification = enums.IecDevId.AUTO) \n
		Selects the mode the instrument identification is performed. \n
			:param identification: AUTO| USER AUTO The *IDN string and the *OPT string are set automatically. USER Enables the selection of user definable *IDN and *OPT strings.
		"""
		param = Conversions.enum_scalar_to_str(identification, enums.IecDevId)
		self._core.io.write(f'SYSTem:IDENtification {param}')



from typing import List

from .......Internal.Core import Core
from .......Internal.CommandsGroup import CommandsGroup
from .......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class SettingCls:
	"""Setting commands group definition. 4 total commands, 1 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("setting", core, parent)

	@property
	def store(self):
		"""store commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_store'):
			from .Store import StoreCls
			self._store = StoreCls(self._core, self._cmd_group)
		return self._store

	def get_catalog(self) -> List[str]:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:SETTing:CATalog \n
		Snippet: value: List[str] = driver.source.bb.arbitrary.mcarrier.setting.get_catalog() \n
		Queries the available settings files in the specified default directory. The settings files are used to set the ARB multi
		carrier submenu. Only files with the file extension *.arb_multcarr will be listed. \n
			:return: catalog: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:MCARrier:SETTing:CATalog?')
		return Conversions.str_to_str_list(response)

	def load(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:MCARrier:SETTing:LOAD \n
		Snippet: driver.source.bb.arbitrary.mcarrier.setting.load(filename = '1') \n
		Loads the settings file. If a settings file with the specified name does not yet exist, it is created. The file extension
		may be omitted. Only files with the file extension *.arb_multcarr will be loaded or created. \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:MCARrier:SETTing:LOAD {param}')

	def clone(self) -> 'SettingCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = SettingCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal import Conversions
from .... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class PowerCls:
	"""Power commands group definition. 25 total commands, 6 Subgroups, 5 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("power", core, parent)

	@property
	def alc(self):
		"""alc commands group. 1 Sub-classes, 2 commands."""
		if not hasattr(self, '_alc'):
			from .Alc import AlcCls
			self._alc = AlcCls(self._core, self._cmd_group)
		return self._alc

	@property
	def attenuation(self):
		"""attenuation commands group. 2 Sub-classes, 1 commands."""
		if not hasattr(self, '_attenuation'):
			from .Attenuation import AttenuationCls
			self._attenuation = AttenuationCls(self._core, self._cmd_group)
		return self._attenuation

	@property
	def limit(self):
		"""limit commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_limit'):
			from .Limit import LimitCls
			self._limit = LimitCls(self._core, self._cmd_group)
		return self._limit

	@property
	def range(self):
		"""range commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_range'):
			from .Range import RangeCls
			self._range = RangeCls(self._core, self._cmd_group)
		return self._range

	@property
	def servoing(self):
		"""servoing commands group. 1 Sub-classes, 6 commands."""
		if not hasattr(self, '_servoing'):
			from .Servoing import ServoingCls
			self._servoing = ServoingCls(self._core, self._cmd_group)
		return self._servoing

	@property
	def level(self):
		"""level commands group. 1 Sub-classes, 0 commands."""
		if not hasattr(self, '_level'):
			from .Level import LevelCls
			self._level = LevelCls(self._core, self._cmd_group)
		return self._level

	# noinspection PyTypeChecker
	def get_lmode(self) -> enums.PowLevMode:
		"""SCPI: [SOURce<HW>]:POWer:LMODe \n
		Snippet: value: enums.PowLevMode = driver.source.power.get_lmode() \n
		Selects the level mode. \n
			:return: lev_mode: NORMal| LOWNoise| LOWDistortion NORM automatic selection of the best settings LNOISe settings for lowest noise LDIStortion settings for lowest distortions
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:POWer:LMODe?')
		return Conversions.str_to_scalar_enum(response, enums.PowLevMode)

	def set_lmode(self, lev_mode: enums.PowLevMode) -> None:
		"""SCPI: [SOURce<HW>]:POWer:LMODe \n
		Snippet: driver.source.power.set_lmode(lev_mode = enums.PowLevMode.LOWDistortion) \n
		Selects the level mode. \n
			:param lev_mode: NORMal| LOWNoise| LOWDistortion NORM automatic selection of the best settings LNOISe settings for lowest noise LDIStortion settings for lowest distortions
		"""
		param = Conversions.enum_scalar_to_str(lev_mode, enums.PowLevMode)
		self._core.io.write(f'SOURce<HwInstance>:POWer:LMODe {param}')

	def get_pep(self) -> float:
		"""SCPI: [SOURce<HW>]:POWer:PEP \n
		Snippet: value: float = driver.source.power.get_pep() \n
		Queries the RF signal peak envelope power. \n
			:return: pep: float
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:POWer:PEP?')
		return Conversions.str_to_float(response)

	def get_power(self) -> float:
		"""SCPI: [SOURce<HW>]:POWer:POWer \n
		Snippet: value: float = driver.source.power.get_power() \n
		Sets the level at the RF output connector. This value does not consider a specified offset.
		The command [:SOURce]:POWer[:LEVel][:IMMediate][:AMPLitude] sets the level of the 'Level' display, that means the level
		containing offset. \n
			:return: amplitude: float Range: -20 to 25
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:POWer:POWer?')
		return Conversions.str_to_float(response)

	def set_power(self, amplitude: float) -> None:
		"""SCPI: [SOURce<HW>]:POWer:POWer \n
		Snippet: driver.source.power.set_power(amplitude = 1.0) \n
		Sets the level at the RF output connector. This value does not consider a specified offset.
		The command [:SOURce]:POWer[:LEVel][:IMMediate][:AMPLitude] sets the level of the 'Level' display, that means the level
		containing offset. \n
			:param amplitude: float Range: -20 to 25
		"""
		param = Conversions.decimal_value_to_str(amplitude)
		self._core.io.write(f'SOURce<HwInstance>:POWer:POWer {param}')

	# noinspection PyTypeChecker
	def get_scharacteristic(self) -> enums.PowLevBehaviour:
		"""SCPI: [SOURce<HW>]:POWer:SCHaracteristic \n
		Snippet: value: enums.PowLevBehaviour = driver.source.power.get_scharacteristic() \n
		Selects the characteristic for the level setting. \n
			:return: characteristic: AUTO| UNINterrupted| CVSWr| USER| MONotone UNINterrupted uninterrupted level setting CVSWr constant-VSWR MONotone strictly monotone
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:POWer:SCHaracteristic?')
		return Conversions.str_to_scalar_enum(response, enums.PowLevBehaviour)

	def set_scharacteristic(self, characteristic: enums.PowLevBehaviour) -> None:
		"""SCPI: [SOURce<HW>]:POWer:SCHaracteristic \n
		Snippet: driver.source.power.set_scharacteristic(characteristic = enums.PowLevBehaviour.AUTO) \n
		Selects the characteristic for the level setting. \n
			:param characteristic: AUTO| UNINterrupted| CVSWr| USER| MONotone UNINterrupted uninterrupted level setting CVSWr constant-VSWR MONotone strictly monotone
		"""
		param = Conversions.enum_scalar_to_str(characteristic, enums.PowLevBehaviour)
		self._core.io.write(f'SOURce<HwInstance>:POWer:SCHaracteristic {param}')

	def get_wignore(self) -> bool:
		"""SCPI: [SOURce]:POWer:WIGNore \n
		Snippet: value: bool = driver.source.power.get_wignore() \n
		Ignores level range warnings. \n
			:return: state: 0| 1| OFF| ON
		"""
		response = self._core.io.query_str('SOURce:POWer:WIGNore?')
		return Conversions.str_to_bool(response)

	def set_wignore(self, state: bool) -> None:
		"""SCPI: [SOURce]:POWer:WIGNore \n
		Snippet: driver.source.power.set_wignore(state = False) \n
		Ignores level range warnings. \n
			:param state: 0| 1| OFF| ON
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'SOURce:POWer:WIGNore {param}')

	def clone(self) -> 'PowerCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = PowerCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class TerrestrialCls:
	"""Terrestrial commands group definition. 36 total commands, 5 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("terrestrial", core, parent)

	@property
	def clock(self):
		"""clock commands group. 0 Sub-classes, 3 commands."""
		if not hasattr(self, '_clock'):
			from .Clock import ClockCls
			self._clock = ClockCls(self._core, self._cmd_group)
		return self._clock

	@property
	def filterPy(self):
		"""filterPy commands group. 2 Sub-classes, 1 commands."""
		if not hasattr(self, '_filterPy'):
			from .FilterPy import FilterPyCls
			self._filterPy = FilterPyCls(self._core, self._cmd_group)
		return self._filterPy

	@property
	def power(self):
		"""power commands group. 0 Sub-classes, 2 commands."""
		if not hasattr(self, '_power'):
			from .Power import PowerCls
			self._power = PowerCls(self._core, self._cmd_group)
		return self._power

	@property
	def symbolRate(self):
		"""symbolRate commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_symbolRate'):
			from .SymbolRate import SymbolRateCls
			self._symbolRate = SymbolRateCls(self._core, self._cmd_group)
		return self._symbolRate

	@property
	def trigger(self):
		"""trigger commands group. 3 Sub-classes, 4 commands."""
		if not hasattr(self, '_trigger'):
			from .Trigger import TriggerCls
			self._trigger = TriggerCls(self._core, self._cmd_group)
		return self._trigger

	def get_delay(self) -> float:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:TERRestrial:DELay \n
		Snippet: value: float = driver.source.bb.xmRadio.terrestrial.get_delay() \n
		No command help available \n
			:return: delay: No help available
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:XMRadio:TERRestrial:DELay?')
		return Conversions.str_to_float(response)

	def set_delay(self, delay: float) -> None:
		"""SCPI: [SOURce<HW>]:BB:XMRadio:TERRestrial:DELay \n
		Snippet: driver.source.bb.xmRadio.terrestrial.set_delay(delay = 1.0) \n
		No command help available \n
			:param delay: No help available
		"""
		param = Conversions.decimal_value_to_str(delay)
		self._core.io.write(f'SOURce<HwInstance>:BB:XMRadio:TERRestrial:DELay {param}')

	def clone(self) -> 'TerrestrialCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = TerrestrialCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ...... import repcap


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class DelayCls:
	"""Delay commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("delay", core, parent)

	def set(self, delay: float, iqConnector=repcap.IqConnector.Default) -> None:
		"""SCPI: [SOURce]:BB:IMPairment:IQOutput<CH>:DELay \n
		Snippet: driver.source.bb.impairment.iqOutput.delay.set(delay = 1.0, iqConnector = repcap.IqConnector.Default) \n
		Defines the time delay of both I and Q vectors between the marker signal at the marker outputs relative to the signal
		generation start. A positive value means that the I and Q vectors delay relative to the marker/trigger or to the other
		instrument and vice versa.
			INTRO_CMD_HELP: The suffix <ch> has the following values: \n
			- <ch>= 0: I/Q Analog Outputs
			- <ch>= 1: I/Q Modulator Digital Impairments \n
			:param delay: float Range: -500E-9 to 500E-9
			:param iqConnector: optional repeated capability selector. Default value: Nr1 (settable in the interface 'IqOutput')
		"""
		param = Conversions.decimal_value_to_str(delay)
		iqConnector_cmd_val = self._cmd_group.get_repcap_cmd_value(iqConnector, repcap.IqConnector)
		self._core.io.write(f'SOURce:BB:IMPairment:IQOutput{iqConnector_cmd_val}:DELay {param}')

	def get(self, iqConnector=repcap.IqConnector.Default) -> float:
		"""SCPI: [SOURce]:BB:IMPairment:IQOutput<CH>:DELay \n
		Snippet: value: float = driver.source.bb.impairment.iqOutput.delay.get(iqConnector = repcap.IqConnector.Default) \n
		Defines the time delay of both I and Q vectors between the marker signal at the marker outputs relative to the signal
		generation start. A positive value means that the I and Q vectors delay relative to the marker/trigger or to the other
		instrument and vice versa.
			INTRO_CMD_HELP: The suffix <ch> has the following values: \n
			- <ch>= 0: I/Q Analog Outputs
			- <ch>= 1: I/Q Modulator Digital Impairments \n
			:param iqConnector: optional repeated capability selector. Default value: Nr1 (settable in the interface 'IqOutput')
			:return: delay: float Range: -500E-9 to 500E-9"""
		iqConnector_cmd_val = self._cmd_group.get_repcap_cmd_value(iqConnector, repcap.IqConnector)
		response = self._core.io.query_str(f'SOURce:BB:IMPairment:IQOutput{iqConnector_cmd_val}:DELay?')
		return Conversions.str_to_float(response)



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from .....Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class CommonCls:
	"""Common commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("common", core, parent)

	def get_hostname(self) -> str:
		"""SCPI: SYSTem:COMMunicate:NETWork:[COMMon]:HOSTname \n
		Snippet: value: str = driver.system.communicate.network.common.get_hostname() \n
		Sets the individual host name of the R&S SGT. Note: it is recommended that you do not change the host name in order to
		avoid problems with the networdk connection. However, if you change the host name be sure to use an unique name. The host
		name is a protected parameter, To change it, first disable protection level 1 with command method RsSgt.System.Protect.
		State.set. \n
			:return: hostname: string
		"""
		response = self._core.io.query_str('SYSTem:COMMunicate:NETWork:COMMon:HOSTname?')
		return trim_str_response(response)

	def set_hostname(self, hostname: str) -> None:
		"""SCPI: SYSTem:COMMunicate:NETWork:[COMMon]:HOSTname \n
		Snippet: driver.system.communicate.network.common.set_hostname(hostname = '1') \n
		Sets the individual host name of the R&S SGT. Note: it is recommended that you do not change the host name in order to
		avoid problems with the networdk connection. However, if you change the host name be sure to use an unique name. The host
		name is a protected parameter, To change it, first disable protection level 1 with command method RsSgt.System.Protect.
		State.set. \n
			:param hostname: string
		"""
		param = Conversions.value_to_quoted_str(hostname)
		self._core.io.write(f'SYSTem:COMMunicate:NETWork:COMMon:HOSTname {param}')



from ....Internal.Core import Core
from ....Internal.CommandsGroup import CommandsGroup
from ....Internal.Types import DataType
from ....Internal.StructBase import StructBase
from ....Internal.ArgStruct import ArgStruct
from ....Internal.ArgSingleList import ArgSingleList
from ....Internal.ArgSingle import ArgSingle


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class ScmdCls:
	"""Scmd commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("scmd", core, parent)

	def set(self, scmd: str, what_is_this: str) -> None:
		"""SCPI: TEST<HW>:SW:SCMD \n
		Snippet: driver.test.sw.scmd.set(scmd = '1', what_is_this = '1') \n
		No command help available \n
			:param scmd: No help available
			:param what_is_this: No help available
		"""
		param = ArgSingleList().compose_cmd_string(ArgSingle('scmd', scmd, DataType.String), ArgSingle('what_is_this', what_is_this, DataType.String))
		self._core.io.write(f'TEST<HwInstance>:SW:SCMD {param}'.rstrip())

	# noinspection PyTypeChecker
	class ScmdStruct(StructBase):
		"""Response structure. Fields: \n
			- Scmd: str: No parameter help available
			- What_Is_This: str: No parameter help available"""
		__meta_args_list = [
			ArgStruct.scalar_str('Scmd'),
			ArgStruct.scalar_str('What_Is_This')]

		def __init__(self):
			StructBase.__init__(self, self)
			self.Scmd: str = None
			self.What_Is_This: str = None

	def get(self) -> ScmdStruct:
		"""SCPI: TEST<HW>:SW:SCMD \n
		Snippet: value: ScmdStruct = driver.test.sw.scmd.get() \n
		No command help available \n
			:return: structure: for return value, see the help for ScmdStruct structure arguments."""
		return self._core.io.query_struct(f'TEST<HwInstance>:SW:SCMD?', self.__class__.ScmdStruct())



from .....Internal.Core import Core
from .....Internal.CommandsGroup import CommandsGroup
from .....Internal import Conversions
from ..... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class SignalCls:
	"""Signal commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("signal", core, parent)

	# noinspection PyTypeChecker
	def get_type_py(self) -> enums.ArbSignType:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:SIGNal:TYPE \n
		Snippet: value: enums.ArbSignType = driver.source.bb.arbitrary.signal.get_type_py() \n
		Selects the type of test signal. \n
			:return: arb_signal_type: SINE| RECT| CIQ
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:SIGNal:TYPE?')
		return Conversions.str_to_scalar_enum(response, enums.ArbSignType)

	def set_type_py(self, arb_signal_type: enums.ArbSignType) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:SIGNal:TYPE \n
		Snippet: driver.source.bb.arbitrary.signal.set_type_py(arb_signal_type = enums.ArbSignType.AWGN) \n
		Selects the type of test signal. \n
			:param arb_signal_type: SINE| RECT| CIQ
		"""
		param = Conversions.enum_scalar_to_str(arb_signal_type, enums.ArbSignType)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:SIGNal:TYPE {param}')



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ......Internal.Utilities import trim_str_response
from ...... import enums


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class WsegmentCls:
	"""Wsegment commands group definition. 23 total commands, 3 Subgroups, 5 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("wsegment", core, parent)

	@property
	def configure(self):
		"""configure commands group. 5 Sub-classes, 5 commands."""
		if not hasattr(self, '_configure'):
			from .Configure import ConfigureCls
			self._configure = ConfigureCls(self._core, self._cmd_group)
		return self._configure

	@property
	def next(self):
		"""next commands group. 1 Sub-classes, 2 commands."""
		if not hasattr(self, '_next'):
			from .Next import NextCls
			self._next = NextCls(self._core, self._cmd_group)
		return self._next

	@property
	def sequence(self):
		"""sequence commands group. 0 Sub-classes, 1 commands."""
		if not hasattr(self, '_sequence'):
			from .Sequence import SequenceCls
			self._sequence = SequenceCls(self._core, self._cmd_group)
		return self._sequence

	def set_cload(self, filename_input: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CLOad \n
		Snippet: driver.source.bb.arbitrary.wsegment.set_cload(filename_input = '1') \n
		Creates a and loads a multi segment waveform using the current entries of the specified configuration file. This multi
		segment waveform is saved with the file name specified in the configuration file. The file extension is *.wv. \n
			:param filename_input: string
		"""
		param = Conversions.value_to_quoted_str(filename_input)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:CLOad {param}')

	def set_create(self, filename_input: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:CREate \n
		Snippet: driver.source.bb.arbitrary.wsegment.set_create(filename_input = '1') \n
		Creates a multi segment waveform using the current settings of the specified configuration file. The multi segment
		waveform is saved into the file defined in the configuration file. The file extension is *.wv. \n
			:param filename_input: string
		"""
		param = Conversions.value_to_quoted_str(filename_input)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:CREate {param}')

	# noinspection PyTypeChecker
	def get_lmode(self) -> enums.ArbLevMode:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:LMODe \n
		Snippet: value: enums.ArbLevMode = driver.source.bb.arbitrary.wsegment.get_lmode() \n
		Sets how the segments are leveled. \n
			:return: level_mode: HIGHest| UNCHanged
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WSEGment:LMODe?')
		return Conversions.str_to_scalar_enum(response, enums.ArbLevMode)

	def set_lmode(self, level_mode: enums.ArbLevMode) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:LMODe \n
		Snippet: driver.source.bb.arbitrary.wsegment.set_lmode(level_mode = enums.ArbLevMode.HIGHest) \n
		Sets how the segments are leveled. \n
			:param level_mode: HIGHest| UNCHanged
		"""
		param = Conversions.enum_scalar_to_str(level_mode, enums.ArbLevMode)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:LMODe {param}')

	def get_name(self) -> str:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:NAME \n
		Snippet: value: str = driver.source.bb.arbitrary.wsegment.get_name() \n
		This command queries the name of the waveform of the currently output segment of the multi segment waveform. \n
			:return: name: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WSEGment:NAME?')
		return trim_str_response(response)

	def get_value(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment \n
		Snippet: value: int = driver.source.bb.arbitrary.wsegment.get_value() \n
		This command queries the currently output segment of the multi segment waveform. \n
			:return: wsegment: integer Range: 0 to 1023
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WSEGment?')
		return Conversions.str_to_int(response)

	def clone(self) -> 'WsegmentCls':
		"""Clones the group by creating new object from it and its whole existing subgroups
		Also copies all the existing default Repeated Capabilities setting,
		which you can change independently without affecting the original group"""
		new_group = WsegmentCls(self._core, self._cmd_group.parent)
		self._cmd_group.synchronize_repcaps(new_group)
		return new_group



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions
from ......Internal.Utilities import trim_str_response


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class SequenceCls:
	"""Sequence commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("sequence", core, parent)

	def get_select(self) -> str:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:SEQuence:SELect \n
		Snippet: value: str = driver.source.bb.arbitrary.wsegment.sequence.get_select() \n
		Selects the sequencing list (files with extension *.wvs) \n
			:return: filename: string
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WSEGment:SEQuence:SELect?')
		return trim_str_response(response)

	def set_select(self, filename: str) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WSEGment:SEQuence:SELect \n
		Snippet: driver.source.bb.arbitrary.wsegment.sequence.set_select(filename = '1') \n
		Selects the sequencing list (files with extension *.wvs) \n
			:param filename: string
		"""
		param = Conversions.value_to_quoted_str(filename)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WSEGment:SEQuence:SELect {param}')



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class HddStreamingCls:
	"""HddStreaming commands group definition. 2 total commands, 0 Subgroups, 2 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("hddStreaming", core, parent)

	def get_blevel(self) -> int:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:HDDStreaming:BLEVel \n
		Snippet: value: int = driver.source.bb.arbitrary.waveform.hddStreaming.get_blevel() \n
		Queries the filling level of the streaming buffer. \n
			:return: blevel: integer Range: 0 to 100
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WAVeform:HDDStreaming:BLEVel?')
		return Conversions.str_to_int(response)

	def get_state(self) -> bool:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:HDDStreaming:STATe \n
		Snippet: value: bool = driver.source.bb.arbitrary.waveform.hddStreaming.get_state() \n
		Enables/disables the streaming of modulation data directly from the hard drive (HDD) . HDD streaming is recommended for
		processing of large files that require more ARB memory than the currently installed one. \n
			:return: state: 0| 1| OFF| ON
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:WAVeform:HDDStreaming:STATe?')
		return Conversions.str_to_bool(response)

	def set_state(self, state: bool) -> None:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:WAVeform:HDDStreaming:STATe \n
		Snippet: driver.source.bb.arbitrary.waveform.hddStreaming.set_state(state = False) \n
		Enables/disables the streaming of modulation data directly from the hard drive (HDD) . HDD streaming is recommended for
		processing of large files that require more ARB memory than the currently installed one. \n
			:param state: 0| 1| OFF| ON
		"""
		param = Conversions.bool_to_str(state)
		self._core.io.write(f'SOURce<HwInstance>:BB:ARBitrary:WAVeform:HDDStreaming:STATe {param}')



from ......Internal.Core import Core
from ......Internal.CommandsGroup import CommandsGroup
from ......Internal import Conversions


# noinspection PyPep8Naming,PyAttributeOutsideInit,SpellCheckingInspection
class MeasureCls:
	"""Measure commands group definition. 1 total commands, 0 Subgroups, 1 group commands"""

	def __init__(self, core: Core, parent):
		self._core = core
		self._cmd_group = CommandsGroup("measure", core, parent)

	def get_state(self) -> bool:
		"""SCPI: [SOURce<HW>]:BB:ARBitrary:CFR:MEASure:[STATe] \n
		Snippet: value: bool = driver.source.bb.arbitrary.cfr.measure.get_state() \n
		Queries the state of the crest factor reduction calculation. \n
			:return: measure_state: 1| ON| 0| OFF ON: the original and resulting crest factors are already calculated.
		"""
		response = self._core.io.query_str('SOURce<HwInstance>:BB:ARBitrary:CFR:MEASure:STATe?')
		return Conversions.str_to_bool(response)
